# -*- coding:utf-8 -*- 
import sys 
from os.path import dirname
from PyQt4.QtCore import (QCoreApplication, QFileInfo, QLocale, QSettings, QTranslator, Qt)
from PyQt4.QtGui import (QAbstractItemView, QAction, QApplication, QBrush, QCalendarWidget, QCheckBox,
                         QColor, QComboBox, QCommandLinkButton, QDialog, QFileDialog, QFont, QGroupBox,
                         QHBoxLayout, QIcon, QImage, QKeySequence, QLabel, QLineEdit, QLinearGradient,
                         QListView, QMenu, QMovie, QPainter, QPixmap, QPrintPreviewDialog, QPrinter,
                         QPushButton, QRadioButton, QSizePolicy, QStandardItem, QStandardItemModel, QTabBar,
                         QTabWidget, QTableView, QTableWidget, QTableWidgetItem, QTextEdit,
                         QTextOption, QToolBar, QToolButton, QWidget)

from qsphere_objmaker import (MyAction, MyButton, MyCheckBox, MyComboBox,
                              MyLabel, MyListView, MyListWidget, MyProgressBar,
                              MyPushButton, MyPushButtonMappable, MyScintilla,
                              MySimpleWidgetLineEdit, MySimpleWidgetLineEditST, MySlider,
                              MySpinBox, MyTableWidget, MyTextEdit, 
                              MyUserComboBox, MyWebComboBox, MyWidgetLineEdit)

from qgis.core import QGis
from doUI import (DialogAbout, DialogOptions, LoadDialogViewer)
from qsphere_tools import (getisocodes_dict, getThemeIcon, getUILangIndex,
                           LoadData, LoadDefautValue, LoadFile,
                           MakeListCountries, makeHelp, MakeListEncoders, MakeListFormats, MakeListLangs,
                           MakeListTemporalSystem, MakeListTypeRessources)

from qsphere_objmaker import MyAction
from ui_mainwindow import myMDIWindow

class MainPlugin(object):
  
  def __init__(self, iface):
    self.name = "qsphere"
    self._objectName = "qsphere"
    self.iface = iface
    self.MainPlugin = self
    self.mainWindow = None
    self.webNavigator = None

    overrideLocale = QSettings().value("locale/overrideFlag", False)
    localeFullName = QLocale.system().name() if not overrideLocale else QSettings().value("locale/userLocale", "")
    self.langueTR = localeFullName[0:2]
    self.localePath = "%s/i18n/qsphere_%s.qm" % (dirname(__file__), self.langueTR)
    self.localeFullName = localeFullName

    reload(sys)
    sys.setdefaultencoding('iso-8859-1') 

    self.languesDico = getisocodes_dict('ISO-639-2_utf-8.txt')
    self.langue, self.languageIndex = getUILangIndex(self)
    self.dicoLangs = ("fr", "en", "it", "de", "es", "fi")
    self.indexLang = self.dicoLangs.index(self.langueTR) if self.langueTR in self.dicoLangs else 0
    self.toolBar = None
    self.translator = QTranslator()

    self.libelles = {
                         0:  ("tab1", QWidget, "Identification", "Identification data (title, abstract,\ntype, encoding ...)", "identification.png", ""), \
                         1:  ("titletab1", QLabel, "Data identification : ","Data identification ", None, ""), \
                         2:  ("intitule", QLineEdit, "Title : ", "Title of spatial data sets and services ", None, "file:intitule_%s.html"), \
                         3:  ("resume", MyTextEdit, "Abstract : ", "Abstract of the dataset describing the resource ", None, "file:resume_%s.html"), \
                         4:  ("typedata", QComboBox, "Resource type : ", "Resource type ", None, "file:type_ressource_%s.html"), \
                         5:  ("tablelocalisator", MyTableWidget, "Locator (s) for the resource : ", "Link(s) for the resource ", None, "file:localisateur_%s.html"), \
                         6:  ("tablelocalisator_c0", None, "URL", "", None, ""), \
                         7:  ("tablelocalisator_c1", None, "Name", "", None, ""), \
                         8:  ("identificator", QLineEdit, "Resource Identifier : ", "Identifier for the resource ", None, "file:identificateur_%s.html"), \
                         9:  ("tablelangues", QTableView, "Language(s) for the resource : ", "List of language(s) for the resource : ", None, "file:langue_%s.html"), \
                         10: ("tableformats", MyTableWidget, "Format(s) : ", "Different formats of spatial data sets and services ", None, "file:format_fichier_%s.html"), \
                         11: ("tableformats_c0", None, "Format", "", None, ""), \
                         12: ("tableformats_c1", None, "", "", None, ""), \
                         13: ("tableformats_c2", None, "Version", "", None, ""), \
                         14: ("tablecarac", QComboBox, "Encoding : ", "Character encoding of spatial data sets and services ", None, "file:encodage_%s.html"), \
                         15: ("tab2", QWidget, "Classification & Keywords","Classification & Keywords", "classification.png", "file:categorie_thematique_%s.html"), \
                         16: ("titletab2_1", QLabel, "Classification of spatial data and services : ", "Data classification and mapping services ", None, ""), \
                         17: ("tablecategories", QTableView, "Thematic categories : ", "Thematic categories\n(INSPIRE Annexes - INSPIRE Themes - ISO classes ) ", None, "file:categorie_thematique_%s.html"), \
                         18: ("titletab2_2", QLabel, "Keywords : ", "Keywords ", None, ""), \
                         19: ("tablemotsclefso", MyTableWidget, "Keywords from INSPIRE<br>Data themes & repositories : ", "List of keywords required (INSPIRE Themes) ", None, "file:mots_clefs_%s.html"), \
                         20: ("tablemotsclefso_c0", None, "Thematic", "", None, ""), \
                         21: ("tablemotsclefso_c1", None, "Keyword", "", None, ""), \
                         22: ("tablemotsclefsf", MyTableWidget, "<i>Free</i> keywords : ", "List of optional keywords (INSPIRE Themes) ", None, "file:mots_clefs_%s.html"), \
                         23: ("tablemotsclefsf_c0", None, "Keyword", "", None, ""), \
                         24: ("tablemotsclefsf_c1", None, "", "", None, ""), \
                         25: ("tablemotsclefsf_c2", None, "Controlled vocabulary ", "", None, ""), \
                         26: ("tablemotsclefsf_c3", None, "Date", "", None, ""), \
                         27: ("tablemotsclefsf_c4", None, "Date type", "", None, ""), \
                         28: ("tab3", QWidget, "Geographic", "Geographical location", "location.png", None, ""), \
                         29: ("titletab3_1", QLabel, "Location : ", "Information for the location (data required) ", None, ""), \
                         30: ("subtitletab3_1", QLabel, "Bouding box : ", "Information for the location (Bouding box) ", None, ""), \
                         31: ("tableemprises", MyTableWidget, "", "List of the extents of spatial data sets and services ", None, "file:emprise_%s.html"), \
                         32: ("tableemprises_c0", None, "North lat.", "", None, ""), \
                         33: ("tableemprises_c1", None, "South lat.", "", None, ""), \
                         34: ("tableemprises_c2", None, "West long.", "", None, ""), \
                         35: ("tableemprises_c3", None, "East long.", "", None, ""), \
                         36: ("tableemprises_c4", None, "Extent", "", None, ""), \
                         37: ("subtitletab3_2", QLabel, "Coordinate reference system(s) : ", "Information about coordinate reference system(s) (data required) ", None, ""), \
                         38: ("tablescr", MyTableWidget, "SRS : " ,"List of reference system(s) of spatial data sets and services ", None, "file:systeme_de_reference_%s.html"), \
                         39: ("tablescr_c0", None, "Selected CRS", "", None, ""), \
                         40: ("tablescr_c1", None, "", "", None, ""), \
                         41: ("tab4", QWidget, "Temporal", "Dates temporal referencing", "temporality.png", ""), \
                         42: ("titletab4_1", QLabel, "Temporal informations : ", "Temporal informations ", None, ""), \
                         43: ("tableetenduetemporelle", MyTableWidget, "Temporal extent : ", "Information about temporal extent (optional data)", None, "file:etendue_temporelle_%s.html"), \
                         44: ("tableetenduetemporelle_c0", None, " Start <-> End ", "", None, ""), \
                         45: ("tableetenduetemporelle_c1", None, "", "", None, ""), \
                         46: ("tabledatepubdata", MyTableWidget, "Date of publication : ", "Date of publication of spatial data sets and services ", None, "file:dates_reference_%s.html"), \
                         47: ("tabledatepubdata_c0", None, "Date", "", None, ""), \
                         48: ("tabledatepubdata_c1", None, "", "", None, ""), \
                         49: ("datecredata", QCalendarWidget, "Date of creation : ", "Date of creation of spatial data sets and services ", None, "file:dates_reference_%s.html"), \
                         50: ("daterevdata", QCalendarWidget, "Date of the last revision : ", "Date of the last revision of spatial data sets and services ", None, "file:dates_reference_%s.html"), \
                         51: ("sysreftemp", QComboBox, "Temporal reference system : ", "Temporal reference system ", None, "file:systeme_de_reference_temporel_%s.html"), \
                         52: ("tab5", QWidget, "Quality, Validity and Conformity", "Quality, Validity and Conformity", "quality.png", ""), \
                         53: ("titletab5_1", QLabel, "Quality & validity : ", "Quality & validity", None, ""), \
                         54: ("subtitletab5_1", QLabel, "Lineage : ", "Lineage of spatial data sets and services ", None, ""), \
                         55: ("genealogie", MyTextEdit, "Lineage for the data : ", "Lineage of spatial data sets and services ", None, "file:genealogie_%s.html"), \
                         56: ("subtitletab5_2", QLabel, "Spatial resolution : ", "Spatial resolution of spatial data sets and services ", None, ""), \
                         57: ("grouperesolutionscale", QGroupBox, "Equivalent scale : ", "Equivalent scale of spatial data sets and services ", None, "file:resolution_spatiale_%s.html"), \
                         58: ("coherence", MyTextEdit, "Topological consistency : ", "Topological consistency of spatial data sets and services (data not required) ", None, "file:coherence_topologique_%s.html"), \
                         59: ("subtitletab5_3", QLabel, "Compliance : ", "Compliance of spatial data sets and services ", None, ""), \
                         60: ("tablespecifications", MyTableWidget, "Compliance specifications : ", "List of compliance specifications of spatial data sets and services ", None, "file:conformite_specifications_%s.html"), \
                         61: ("tablespecifications_c0", None, "Specification", "", None, ""), \
                         62: ("tablespecifications_c1", None, "Date", "", None, ""), \
                         63: ("tablespecifications_c2", None, "Date type", "", None, ""), \
                         64: ("tablespecifications_c3", None, "Class", "", None, ""), \
                         65: ("tab6", QWidget, "Constraints", "Access and uses", "rights_and_uses.png", ""), \
                         66: ("titletab6_1", QLabel, "Constraint related to access and use : ", "Constraint related to access and use ", None, ""), \
                         67: ("groupedroits", QGroupBox, "Conditions applying...\nto access and use : ", "Conditions applying to access and use ", None, "file:contraintes_%s.html"), \
                         68: ("licence", MyTextEdit, "Conditions licensing applying...\nto access and use : ", "Licensing conditions applying to access and use ", None, "file:licence_%s.html"), \
                         69: ("tab7", QWidget, "Responsible party", "Responsible party", "responsible.png", ""), \
                         70: ("titletab7_1", QLabel, "Responsible party : ", "Organisations reponsible for the etablishment, management, maintenance and distribution of spatial data sets and services ", None, ""), \
                         71: ("tableroles", MyTableWidget, "", "Responsible party role ", None, "file:organisme_responsable_%s.html"), \
                         72: ("tableroles_c0", None, "Role", "", None, ""), \
                         73: ("tableroles_c1", None, "Organization name", "", None, ""), \
                         74: ("tableroles_c2", None, "Address", "", None, ""), \
                         75: ("tableroles_c3", None, "Country", "", None, ""), \
                         76: ("tableroles_c4", None, "Zip code", "", None, ""), \
                         77: ("tableroles_c5", None, "City", "", None, ""), \
                         78: ("tableroles_c6", None, "E-mail", "", None, ""), \
                         79:("tableroles_c7", None, "Phone", "", None, ""), \
                         80:("tableroles_c8", None, "URL", "", None, ""), \
                         81: ("tab8", QWidget, "Metadata", "Metadata", "metadata.png", ""), \
                         82: ("titletab8_1", QLabel, "Informations about Metadata : ", "Informations about Metadata ", None, ""), \
                         83: ("datemetada", QCalendarWidget, "Metadata date : ", "Metadata date ", None, "file:date_metadonnee_%s.html"), \
                         84: ("langmetada", QComboBox, "Metadata language : ", "Metadata language ", None, "file:langue_metadonnee_%s.html"), \
                         85: ("tab9", QWidget, "QGIS metadata", "Metadata by QGIS", "qgis.png", ""), \
                         86: ("titletab9_1", QLabel, "Additional Information QGIS QSPHERE : ", "Additional Information QGIS QSPHERE : ", None, ""), \
                         87: ("namelayer", QLineEdit, "Name layer : ", "Name Layer ", None, "file:catalogue_attributaire_%s.html"), \
                         88: ("typelayer", QLineEdit, "Type layer : ", "Type Layer ", None, "file:catalogue_attributaire_%s.html"), \
                         89: ("metadata", MyTextEdit, "", "Generic metadata QGIS for geographical layer (vector or raster) ", None, "file:catalogue_attributaire_%s.html"), \
                         90: ("tabledico", MyTableWidget, "", "Fields for the query layer (vector) ", None, "file:catalogue_attributaire_%s.html") , \
                         91: ("tabledico_c0", None, "Id", "", None, ""), \
                         92: ("tabledico_c1", None, "Name", "", None, ""), \
                         93: ("tabledico_c2", None, "Type", "", None, ""), \
                         94: ("tabledico_c3", None, "Length", "", None, ""), \
                         95: ("tabledico_c4", None, "Precision", "", None, ""), \
                         96: ("tabledico_c5", None, "Comment", "", None, ""), \
                         100:("helptopic", MyPushButton, "", "Get informations for this topic", "info.png", ""), \
                         101:("actionsmetadata", MyPushButton, "", "Actions for the metadata...", "actions.png", ""), \
                         102:("callwizard", MyPushButton, "", "Call the wizards ...", "wizards_form.png", "")
                         }

    self.formats = MakeListFormats(self)
    self.langs = MakeListLangs(self)
    self.listCodecs = MakeListEncoders(self)
    self.listTemporalSystem = MakeListTemporalSystem(self)
    self.listTypeRessources = MakeListTypeRessources(self)
    self.listCountries, self.indexCountry, self.listCountriesCode = MakeListCountries(self)

    self.ParamsLineWidget = {
                                  "tablescr" : [(0, 3, "4258"), (1, -1, "")], \
                                  "tableformats" : [(0, 6, ""), ( 1, -1, ""), (-1, 0, "")], \
                                  "tableroles" : [(2, 0, 6),(-1, 0, ""), (-1, 0, ""), (2, 5, self.indexCountry), (0, 2, ""), (-1, 0, ""), (0, 0, ""), (-1, 0, ""), (0, 4, "")], \
                                  "tablelocalisator" : [(0, 4, ""), (-1, 0, "")], \
                                  "tableemprises" : [(3, 0, -1), (3, 0, -1), (3, 0, -1), (3, 0, -1), (5, 0, "")], \
                                  "tableetenduetemporelle" :[(0, 1, "")], \
                                  "tabledatepubdata": [(0, 5, "")], \
                                  "tablemotsclefso" : [(2, 1, 0), (2, -1, (0,0))], \
                                  "tablemotsclefsf" : [(-1, 0, ""), (6, 0, 0), (-1, 0, ""), (0, 5, ""), (2, 2, 0)], \
                                  "tablespecifications" : [(-1, 0, ""), (0, 5, ""), (2, 2, 0), (2, 3, 0)], \
                                  "tableechelles" : [(4, 0, -1)], \
                                  "tabledico" : [(-1, 0, ""), (-1, 0, ""), (-1, 0, ""), (-1, 0, ""), (-1, 0, ""), (-1, 0, "")]
                                  }


    
    self.changeLang(False)

  def objectName(self): return self._objectName
  def changeLang(self, toReload):
        overrideLocale = QSettings().value("locale/overrideFlag", False)
        localeFullName = QLocale.system().name() if not overrideLocale else QSettings().value("locale/userLocale", "")
        self.localePath = "%s/i18n/qsphere_%s.qm" % (dirname(__file__), self.dicoLangs[self.indexLang])
        self.ListOfRules, self.DicoListOfRules = LoadData(self, "file:/ressources/roles_%s.csv" % (self.dicoLangs[self.indexLang]), "roles")

        if QFileInfo(self.localePath).exists():
             self.translator.load(self.localePath)
             QCoreApplication.installTranslator(self.translator)

        if toReload : self.translate()


  def initGui(self):
    self.menu = QMenu("QSphere")
    self.menu.setObjectName("mQsphereMenu")

    self.barInfo = self.iface.messageBar()
    self.barInfo.setSizePolicy( QSizePolicy.Minimum, QSizePolicy.Fixed )
    self.barInfo.setGeometry(0, 0, self.iface.mapCanvas().width()-10, 60)

    try: self.QgisVersion =  QGis.QGIS_VERSION_INT 
    except: self.QgisVersion = QGis.qgisVersion[0]

    if self.toolBar == None :
      if self.QgisVersion < 20400 : self.toolBar = self.iface.addToolBar("QSphere");
      else : self.toolBar = QToolBar("QSphere"); self.iface.addToolBar(self.toolBar);
      self.toolBar.setObjectName("mQsphereToolBar")   

    self.mainWindowAction = MyAction("mActionMainWindow", "", getThemeIcon("qspheremainui.png"), self.iface.mainWindow())
    self.mainNavigator = MyAction("mActionMainNavigator", "", getThemeIcon("navigatorweb.png"), self.iface.mainWindow())
    self.options = MyAction("mActionOptions", "", getThemeIcon("qsphereoptions.png"), self.iface.mainWindow())
    self.help = MyAction("mActionHelp", "", getThemeIcon("help.png"), self.iface.mainWindow())
    self.about = MyAction("mActionAbout", "", getThemeIcon("about.png"), self.iface.mainWindow())

    self.translate()

    self.menu.addAction(self.mainWindowAction)
    self.menu.addSeparator()
    self.menu.addAction(self.mainNavigator)
    self.menu.addAction(self.options)
    self.menu.addSeparator()
    self.menu.addAction(self.help)
    self.menu.addAction(self.about)

    menuBar = self.iface.mainWindow().menuBar()
    zMenu = menuBar
    for child in menuBar.children():
        if child.objectName()== "mPluginMenu" : zMenu =  child; break;
    zMenu.addMenu(self.menu)

    self.toolBar.addAction(self.mainWindowAction)
    self.toolBar.addSeparator()
    self.toolBar.addAction(self.mainNavigator)
    self.toolBar.addAction(self.options)
    self.toolBar.addSeparator()    
    self.toolBar.addAction(self.help)
    self.toolBar.addAction(self.about)

    self.mainWindowAction.triggered.connect(self.clickMainWindow)
    self.mainNavigator.triggered.connect(self.clickNavigator)
    self.options.triggered.connect(self.clickOptions)
    self.help.triggered.connect(self.clickHelp)
    self.about.triggered.connect(self.clickAbout)


  def translate(self):
      self.mainWindowAction.setText(QApplication.translate("QSphere","Open MainWindow", None, QApplication.UnicodeUTF8))
      self.mainNavigator.setText(QApplication.translate("QSphere","Web navigator ...", None, QApplication.UnicodeUTF8))
      self.options.setText(QApplication.translate("QSphere","Call the options GUI ...", None, QApplication.UnicodeUTF8))
      self.help.setText(QApplication.translate("QSphere","Call the help page", None, QApplication.UnicodeUTF8))
      self.about.setText(QApplication.translate("QSphere","About ...", None, QApplication.UnicodeUTF8))


      self.ListOfConformities = LoadDefautValue(self, "file:/ressources/conformity_default_%s.csv"  % (self.dicoLangs[self.indexLang]))
      self.ListOfRules, self.DicoListOfRules = LoadData(self, "file:/ressources/roles_%s.csv" % (self.dicoLangs[self.indexLang]), "roles") 
      self.ListOfThesaurus , self.DateListOfThesaurus = LoadData(self, "file:/ressources/thesaurus_%s.csv" % (self.dicoLangs[self.indexLang]), "thesaurus")
      
      self.ListTypeDates = [QApplication.translate("QSphere","Date of creation", None, QApplication.UnicodeUTF8), \
                            QApplication.translate("QSphere","Date of last revision", None, QApplication.UnicodeUTF8), \
                            QApplication.translate("QSphere","Date of publication", None, QApplication.UnicodeUTF8),]
      self.ListDegres = [QApplication.translate("QSphere","Conformant", None, QApplication.UnicodeUTF8), \
                         QApplication.translate("QSphere","Not conformant", None, QApplication.UnicodeUTF8), \
                         QApplication.translate("QSphere","Not evaluated", None, QApplication.UnicodeUTF8)]

      self.zTablesWidget = {"tablelocalisator" : ((190, 200), \
                              (QApplication.translate("QSphere", self.libelles[6][2], None, QApplication.UnicodeUTF8), \
                               QApplication.translate("QSphere", self.libelles[7][2], None, QApplication.UnicodeUTF8)) ), \
                           "tableformats" : ((80, 40, 80), \
                              (QApplication.translate("QSphere", self.libelles[11][2], None, QApplication.UnicodeUTF8), \
                               QApplication.translate("QSphere", self.libelles[12][2], None, QApplication.UnicodeUTF8), \
                               QApplication.translate("QSphere", self.libelles[13][2], None, QApplication.UnicodeUTF8)) ), \
                           "tableemprises" : ((140, 140, 140, 140, 80), \
                              (QApplication.translate("QSphere", self.libelles[32][2], None, QApplication.UnicodeUTF8), \
                               QApplication.translate("QSphere", self.libelles[33][2], None, QApplication.UnicodeUTF8), \
                               QApplication.translate("QSphere", self.libelles[34][2], None, QApplication.UnicodeUTF8), \
                               QApplication.translate("QSphere", self.libelles[35][2], None, QApplication.UnicodeUTF8), \
                               QApplication.translate("QSphere", self.libelles[36][2], None, QApplication.UnicodeUTF8)) ), \
                           "tablescr" : ((120, 40), \
                              (QApplication.translate("QSphere", self.libelles[39][2], None, QApplication.UnicodeUTF8), \
                               QApplication.translate("QSphere", self.libelles[40][2], None, QApplication.UnicodeUTF8)) ), \
                           "tableetenduetemporelle" : ((140), \
                              (QApplication.translate("QSphere", self.libelles[44][2], None, QApplication.UnicodeUTF8), \
                               QApplication.translate("QSphere", self.libelles[45][2], None, QApplication.UnicodeUTF8)) ), \
                           "tabledatepubdata" : ( (100), \
                              (QApplication.translate("QSphere", self.libelles[47][2], None, QApplication.UnicodeUTF8), \
                               QApplication.translate("QSphere", self.libelles[48][2], None, QApplication.UnicodeUTF8)) ), \
                            "tablemotsclefso" : ((200, 300), \
                              (QApplication.translate("QSphere", self.libelles[20][2], None, QApplication.UnicodeUTF8), \
                               QApplication.translate("QSphere", self.libelles[21][2], None, QApplication.UnicodeUTF8)) ), \
                           "tablemotsclefsf" : ((150, 25, 140, 80, 90), \
                              (QApplication.translate("QSphere", self.libelles[23][2], None, QApplication.UnicodeUTF8), \
                               QApplication.translate("QSphere", self.libelles[24][2], None, QApplication.UnicodeUTF8), \
                               QApplication.translate("QSphere", self.libelles[25][2], None, QApplication.UnicodeUTF8), \
                               QApplication.translate("QSphere", self.libelles[26][2], None, QApplication.UnicodeUTF8), \
                               QApplication.translate("QSphere", self.libelles[27][2], None, QApplication.UnicodeUTF8)) ), \
                            "tablespecifications" : ((240, 80, 100, 100),\
                              (QApplication.translate("QSphere", self.libelles[61][2], None, QApplication.UnicodeUTF8), \
                               QApplication.translate("QSphere", self.libelles[62][2], None, QApplication.UnicodeUTF8), \
                               QApplication.translate("QSphere", self.libelles[63][2], None, QApplication.UnicodeUTF8), \
                               QApplication.translate("QSphere", self.libelles[64][2], None, QApplication.UnicodeUTF8))), \
                           "tableroles" : ((100, 80, 80, 80, 60, 80, 80, 80, 80), \
                              (QApplication.translate("QSphere", self.libelles[72][2], None, QApplication.UnicodeUTF8), \
                               QApplication.translate("QSphere", self.libelles[73][2], None, QApplication.UnicodeUTF8), \
                               QApplication.translate("QSphere", self.libelles[74][2], None, QApplication.UnicodeUTF8), \
                               QApplication.translate("QSphere", self.libelles[75][2], None, QApplication.UnicodeUTF8), \
                               QApplication.translate("QSphere", self.libelles[76][2], None, QApplication.UnicodeUTF8), \
                               QApplication.translate("QSphere", self.libelles[77][2], None, QApplication.UnicodeUTF8), \
                               QApplication.translate("QSphere", self.libelles[78][2], None, QApplication.UnicodeUTF8), \
                               QApplication.translate("QSphere", self.libelles[79][2], None, QApplication.UnicodeUTF8), \
                               QApplication.translate("QSphere", self.libelles[80][2], None, QApplication.UnicodeUTF8)) ), \
                           "tabledico" : ((50, 150, 70, 60, 50, 150), \
                              (QApplication.translate("QSphere", self.libelles[91][2], None, QApplication.UnicodeUTF8), \
                               QApplication.translate("QSphere", self.libelles[92][2], None, QApplication.UnicodeUTF8), \
                               QApplication.translate("QSphere", self.libelles[93][2], None, QApplication.UnicodeUTF8), \
                               QApplication.translate("QSphere", self.libelles[94][2], None, QApplication.UnicodeUTF8), \
                               QApplication.translate("QSphere", self.libelles[95][2], None, QApplication.UnicodeUTF8), \
                               QApplication.translate("QSphere", self.libelles[96][2], None, QApplication.UnicodeUTF8)) ), \
                           "tableechelles" : ((70, ), \
                               ("",""))}
    
  def clickOptions(self): self.dialogOptions = DialogOptions(self); self.dialogOptions.exec_();
       
  def clickHelp(self): makeHelp(self)

  def clickAbout(self): self.dialogAbout = DialogAbout(); self.dialogAbout.exec_();

  def clickNavigator(self):
      if not self.webNavigator : self.webNavigator = LoadDialogViewer(self, self.iface, "#blank", False, [], self.langueTR, self, None, "navigatorweb.png", False)
      self.webNavigator.setWindowState(Qt.WindowMaximized)
      self.webNavigator.show()
      
  def clickMainWindow(self):
      if not self.mainWindow : self.mainWindow = myMDIWindow(self.MainPlugin)
      self.mainWindow.setWindowState(Qt.WindowMaximized)
      self.mainWindow.show()
      
  def unload(self): pass


