from PyQt4.QtCore import (QRect, QSettings, QSize, Qt)
from PyQt4.QtGui import (QAction, QApplication, QButtonGroup, QFileDialog, QFont, QFrame, QGroupBox,
                         QHBoxLayout, QIcon, QImage, QKeySequence, QLabel, QLineEdit, QListWidget,
                         QListWidgetItem, QMainWindow, QMdiArea, QMenu, QMessageBox, QMovie, QPixmap,
                         QProgressBar, QPushButton, QScrollArea, QSizePolicy, QTabWidget, QTextEdit, QToolBar, QWidget)
from qgis.gui import QgsMessageBar
from math import ceil as ceil 
from os.path import (basename, dirname, exists, splitext)
from doUI import (DialogAbout, DialogContacts, DialogMetadata, DialogOptions, LoadDialogViewer)
from qsphere_tools import (FixeLabelsFileDialog, GetLayerCombo, GetExtension, LoadFile, MakePropertiesForWindow,
                           MakeWindowIcon, SendMessage, appendNameSpace, getSelConnexion,
                           getThemeIcon, makeGetOptions, makeHelp)
from qsphere_objmaker import (
        MyPushButton, AuthCSWTDialog, LoadFromLayersDialog,
        mySubWindow, LusTREDialog, toolSearchFilesDialog)
from cswt_connector import cswtConnection
from ui_editorXML import xmlEditor
from importexport import (CloseLOG, ExportToXML, MakeDateStampXML, SaveQSP, WriteInLOG)
from xml.etree import ElementTree as ET
from cStringIO import StringIO
from webbrowser import open as webbrowseropen


class myMDIWindow(QMainWindow):
        def __init__(self, parent): 
            QMainWindow.__init__(self) 
            
            self.setObjectName("qsphereMainWindow")
            self.setAccessibleName("qsphereMainWindow")            
            self.setWindowIcon(QIcon(getThemeIcon("qspheremainui.png")))
            self.racTitle = "QSphere 4.2.0"
            self.setWindowTitle(self.racTitle)
            self.setWindowFlags(Qt.WindowMaximizeButtonHint | Qt.WindowMinimizeButtonHint)
            self.setStyleSheet("QMdiSubWindow {color: white; background-color: qlineargradient(x1:0, y1:0, x2:0, y2:1, stop: 0 #cccccc, stop: 1 #5D5D5D);}")
            self.resize(QSize(QRect(50,50, 1100, 800).size()).expandedTo(self.minimumSizeHint()))

            self.dicoLabel = {}
            self.extension = "qsp"
            self.counterNew = 0
            self.currentSession = None

            self.mdi = QMdiArea()
            self.mdi.setTabsClosable(True)
            self.mdi.setTabsMovable(True)
            
            self.parent = parent
            self.MainPlugin = self.parent.MainPlugin
            self._connexionCSWT = None
            self.webNavigator = None
            self.dialogLUSTRE = None
            self.dialogSearchFiles = None
            self.listSessions = []
            self.listMDDFiles = []

            MakePropertiesForWindow(self, parent)
            makeGetOptions(self)

            self.listSubWindows = []
            self.windowsNoMDIChilds = []

            self.status_txt = QLabel(self.mdi)
            self.movie = QMovie(getThemeIcon("sablier.gif"))
            self.status_txt.setMovie(self.movie)
            self.status_txt.setLayout(QHBoxLayout())
            self.status_txt.layout().addWidget(QLabel(''))
            self.status_txt.setVisible(True)

            self.listNameActionsTabEdition = [ "identificationTab", "classificationTab", "locationTab", "temporalityTab", \
                                               "qualityTab", "rights_and_usesTab", "responsibleTab", "metadataTab", "qgisTab"]

            self.listNameActionsTabWebServices = ["authCSWTButton", "closeCSWTButton", "reportingCSWTButton", "recordsCSWTButton", \
                                         "addCSWTButton", "editCSWTButton", "delCSWTButton"]
            
            self.styleSheetPushButton = "QPushButton::checked {icon-size: 48px, 48px; border-width: 5px; border-radius: 10px; border-color: black; background-color: #9B9B9B;}"
            self.styleSheetPushButton+= "QPushButton {icon-size: 48px, 48px; border-width: 0px; border-radius: 10px; }"

            self.InitDir = dirname(__file__)
            
            self.setCentralWidget(self.mdi)
            
            self.tabWidget = QTabWidget(self)
            self.tabWidget.setObjectName("tabWidget")
            self.tabWidget.setTabShape(QTabWidget.Rounded)
            self.tabWidget.setDocumentMode(True)            
            self.tabWidget.setIconSize(QSize(64, 64))

            self.setMenuWidget(self.tabWidget)
            self.menuWidget().setMinimumHeight(120)
            self.menuWidget().setMaximumHeight(120)


            self.tabFile = QWidget(self.tabWidget)
            self.tabFile.setObjectName("tabFile") 
            self.tabFile.setAccessibleName("tabFile")

            self.loadInfos("sessions", self.listSessions)
            self.loadInfos("fileshistory", self.listMDDFiles) 
            
            self.listActionsTabFile = [
            ("~" , "Sessions", "sessions.png", (10, 0, 280, 40), None, "", True),              
            ("newSession", "New session", "new.png", (100, 5, 32, 32), self.newSession, "", True),       
            ("openSession", "Load a session QSphere", "open.png", (125, 5, 50, 32), "self.buildContextMnuSession(Obj)", "", True),
            ("saveSession", "Save current session", "save.png", (180, 5, 32, 32), self.saveSession, "", False),
            ("saveSessionAs", "Actions for the sessions", "saveactions.png", (215, 5, 50, 32), "self.contextMnuSaveSessionActions(Obj)", "", True),
            ("-", "-", "", (260, 10, 1, 40), None, "", True),
            ("~" , "Files", "files.png", (10, 0, 440, 40), None, "", True), 
            ("newFile", "New sheet", "new.png", (100, 5, 32, 32), self.newMetadataFile, "", True),
            ("openFile", "Load a metadata file", "open.png", (125, 5, 50, 32), "self.buildContextMnuFile(Obj)", "", True),
            ("openFromLayers", "Load metadata file for layers", "openfromlayers.png", (190, 5, 32, 32), self.openMetadataFileForLayers, "Ctrl+L", True),
            ("openFromDataBase", "Load metadata file from database", "openfromdatabase.png", (235, 5, 32, 32), None, "Ctrl+D", False),
            ("saveFile", "Save the current sheet QSphere", "save.png", (285, 5,  32, 32), self.saveMetadataFile, "Ctrl+S", False),
            ("saveFileAs", "Actions for the metadata...", "saveactions.png", (320, 5, 50, 32), "self.contextMnuSaveActions(Obj)", "", True),
            ("saveAll", "Save all files", "saveall.png", (380, 5, 32, 32), self.saveAllMetadataFile, "", False),
            ("-", "-", "", (715, 10, 1, 40), None, "", True),
            ("~" , "Close", "files.png", (10, 0, 160, 40), None, "", True),
            ("close", "Close", "qspherequit.png", (100, 5, 32, 32), self.close, "ALT+F4", True)            
            ]
            toolBar = self.makeMenuCentralWidget(self.tabFile, "tabfiles.png", "Resources",  self.listActionsTabFile)

            self.tabEdition = QWidget(self.tabWidget)
            self.tabEdition.setObjectName("tabEdition") 
            self.tabEdition.setAccessibleName("tabEdition") 
            self.positionTab = QButtonGroup(self.tabEdition)
            self.positionTab.setExclusive(True)
            self.changeMode = QButtonGroup(self.tabEdition)
            self.changeMode.setExclusive(True)
  
            self.listActionsTabEdition = [
            ("~" , "Tabs", "sessions.png", (10, 0, 440, 40), None, "", True),        
            ("identificationTab", "Identification data (title, abstract,\ntype, encoding ...)", "identification.png", (100, 5, 32, 32), self.fixeMetaTab, "", True),
            ("classificationTab", "Classification & Keywords", "classification.png", (135, 5, 32, 32), self.fixeMetaTab, "", True),
            ("locationTab", "Geographical location", "location.png", (170, 5, 32, 32), self.fixeMetaTab, "", True),
            ("temporalityTab", "Dates temporal referencing", "temporality.png", (205, 5, 32, 32), self.fixeMetaTab, "", True),
            ("qualityTab", "Quality, Validity and Conformity", "quality.png", (240, 5, 32, 32), self.fixeMetaTab, "", True),
            ("rights_and_usesTab", "Access and uses", "rights_and_uses.png", (275, 5, 32, 32), self.fixeMetaTab, "", True),
            ("responsibleTab", "Responsible party", "responsible.png", (310, 5, 32, 32), self.fixeMetaTab, "", True),
            ("metadataTab", "Metadata", "metadata.png", (345, 5, 32, 32), self.fixeMetaTab, "", True),
            ("qgisTab", "Metadata by QGIS", "qgis.png", (385, 5, 32, 32), self.fixeMetaTab, "", True),
            ("-", "-", "", (585, 10, 1, 40), None, "", True),
            ("~" , "Modes", "sessions.png", (10, 0, 230, 40), None, "", True),
            ("formMode", "Form view", "n_project.png", (100, 5, 32, 32), None, "", True),
            ("xmlMode", "rendering XML", "n_xml.png", (135, 5, 32, 32), None, "", True),
            ("htmlMode", "rendering HTML", "n_html.png", (170, 5, 32, 32), None, "", True),
            ("-", "-", "", (780, 10, 1, 40), None, "", True),
            ("~" , "Actions", "sessions.png", (10, 0, 190, 40), None, "", True),
            ("reloadWindow", "Reload the current metadata file", "reload.png", (100, 5, 32, 32), self.reloadWindow, "F5", True),
            ("printTab", "Print current page", "print.png", (135, 5, 32, 32), self.printMetaTab, "Ctrl+P", True),
            ("-", "-", "", (910, 10, 1, 40), None, "", True),
            ("~" , "Wizards", "sessions.png", (10, 0, 220, 40), None, "", True),
            ("wizForm", "Call the form wizard Tables ...", "wizform.png", (100, 5, 32, 32), self.openFormWizardMode, "", False),
            ("wizEditorXML", "Edit XML", "wizeditxml.png", (135, 5, 32, 32), self.openXMLEditorWizardMode, "", False),
            ("wizContacts", "Contacts manager ...", "wizcontacts.png", (170, 5, 32, 32), self.openContactsWizardMode, "", False)
            ]
            toolBar = self.makeMenuCentralWidget(self.tabEdition, "tabedit.png", "Edition",  self.listActionsTabEdition)

            self.changeMode.buttonClicked[int].connect(self.changeMetadataMode)

            self.tabWebServices = QWidget(self.tabWidget)
            self.tabWebServices.setObjectName("tabWebServices") 
            self.tabWebServices.setAccessibleName("tabWebServices") 

            self.listActionsTabWebServices = [
            ("~" , "Web", "sessions.png", (10, 0, 170, 40), None, "", True),                    
            ("navigatorWeb", "Web navigator ...", "servers.png", (100, 5, 50, 32), "self.buildContextMnuNavigatorWeb(Obj)", "", True),
            ("-", "-", "", (65, 10, 1, 40), None, "", True),
            ("~" , "Lustre", "sessions.png", (10, 0, 170, 40), None, "", True), 
            ("openLustreWizard", "Call the LusTRE wizard ...", "lustre_wizard.png", (100, 5, 32, 32), self.openLustreWizard, "", True),
            ("-", "-", "", (130, 10, 1, 40), None, "", True),
            ("~" , "CSWT", "sessions.png", (10, 0, 420, 40), None, "", True),
            ("authCSWTButton", "Authentification for the CSW-T Server ...", "connect.png", (100, 5, 32, 32), self.authCSWT, "", True),
            ("closeCSWTButton", "Close the CSWT session", "disconnect.png", (140, 5, 32, 32), self.disconnectCSWT, "", False),
            ("reportingCSWTButton", "Get all reporting for CSWT session", "reporting.png", (190, 5, 32, 32), self.getAllReporting, "", False),
            ("recordsCSWTButton", "Get records in CSWT server", "records.png", (230, 5, 32, 32), self.recordsInCSWT, "", False),
            ("addCSWTButton", "Add to the CSW-T Server", "cloudadd.png", (280, 5, 32, 32), self.insertInCSWT, "", False),
            ("editCSWTButton", "Update to the CSW-T Server", "cloudupdate.png", (320, 5, 32, 32), self.updateInCSWT, "", False),
            ("delCSWTButton", "Delete to the CSW-T Server", "clouddel.png", (360, 5, 32, 32), self.deleteInCSWT, "", False)
            ]
            toolBar = self.makeMenuCentralWidget(self.tabWebServices, "tabwebservices.png", "Web and Services",  self.listActionsTabWebServices)
            
            self.tabWindows = QWidget(self.tabWidget)
            self.tabWindows.setObjectName("tabWindows") 
            self.tabWindows.setAccessibleName("tabWindows")

            self.listActionsTabWindows = [
            ("~" , "Spy", "sessions.png", (10, 0, 260, 40), None, "", True),        
            ("windowsCount", "Sub mainwindow's", "windowscounts.png", (100, 5, 32, 32), None, "", True),
            ("childsCount", "Metadata window's children", "windowchilds.png", (170, 5, 32, 32), None, "", True),
            ("-", "-", "", (145, 10, 1, 40), None, "", True),
            ("~" , "Organize", "sessions.png", (10, 0, 240, 40), None, "", True), 
            ("tileWindows", "Tile windows", "windowstile.png", (100, 5, 32, 32), self.tileWindows, "", True),
            ("cascadeWindows", "Cascade windows", "windowscascade.png", (140, 5, 32, 32), self.cascadeWindows, "", True),
            ("batteriesWindows", "Batteries windows", "windowsbatteries.png", (180, 5, 32, 32), self.batteriesWindows, "", True),
            ("-", "-", "", (325, 10, 1, 40), None, "", True),
            ("~" , "Close", "sessions.png", (10, 0, 190, 40), None, "", True), 
            ("closeWindow", "Close active window", "windowclose.png", (100, 5, 32, 32), self.closeCurrentWindow, "", False),
            ("closeAllWindows", "Close all windows", "windowscloseall.png", (135, 5, 32, 32), self.closeAllMySubWindows, "", False),
            ("-", "-", "", (455, 10, 1, 40), None, "", True),
            ("~" , "Widget", "sessions.png", (10, 0, 160, 40), None, "", True),            
            ("modeWindow", "View / Mask widget control for windows", "windowmdiarea_false.png", (100, 5, 32, 32), self.changeViewWindowsButtons, "", True)
            ]
            toolBar = self.makeMenuCentralWidget(self.tabWindows, "tabwindows.png", "Windows",  self.listActionsTabWindows)
            
            self.tabToolsOptions = QWidget(self.tabWidget)
            self.tabToolsOptions.setObjectName("tabToolsOptions") 
            self.tabToolsOptions.setAccessibleName("tabToolsOptions") 

            self.listActionsTabOptions = [
            ("~" , "Tools", "sessions.png", (10, 0, 240, 40), None, "", True),        
            ("openToolSearchFiles", "Open search files tool", "toolssearchfiles.png", (100, 5, 32, 32), self.openToolSearchFiles, "", True),
            ("openXMLEditor", "Edit XML", "editxml.png", (140, 5, 32, 32), self.openXMLEditor, "", True),                         
            ("openContacts", "Contacts manager ...", "contact.png", (185, 5, 32, 32), self.openContacts, "", True),
            ("-", "-", "", (190, 10, 1, 40), None, "", True),
            ("~" , "Params", "sessions.png", (10, 0, 160, 40), None, "", True),
            ("openOptions", "Call the options GUI ...", "qsphereoptions.png", (100, 5, 32, 32), self.openOptions, "", True),
            ("-", "-", "", (190, 10, 1, 40), None, "", True),
            ("~" , "Help", "sessions.png", (10, 0, 160, 40), None, "", True),
            ("openHelp", "Call the help page", "help.png", (100, 5, 32, 32), self.openHelp,"F1", True)
            ]
            toolBar = self.makeMenuCentralWidget(self.tabToolsOptions, "tabtoolsparams.png", "Tools and Params",  self.listActionsTabOptions)

            self.blocProgressBar = QLabel(self)    
            zIcon = QImage(getThemeIcon("backgroundprogressbar.png"))
            self.blocProgressBar.setPixmap(QPixmap.fromImage(zIcon))
            self.blocProgressBar.setVisible(False)

            self.blocProgressBarButton = MyPushButton(self.blocProgressBar)
            self.blocProgressBarButton.initPushButton(40, 40, 20, 85, "blocProgressBarButton", "", "", True, getThemeIcon(""), 40, 40, True)            

            font = QFont()
            font.setFamily('Courier')
            font.setFixedPitch(True)
            font.setPointSize(10)
            font.setBold(False)

            self.blocProgressBarText = QLabel(self.blocProgressBar)
            self.blocProgressBarText.setStyleSheet("QLabel { font-size: 10px; color : black; background-color: rgba(131, 131, 131, 125); border: 2px solid cyan;  border-radius: 4px;  padding: 2px;}")
            self.blocProgressBarText.setGeometry(5, 180, 390, 40)
            self.blocProgressBarText.setAlignment(Qt.AlignCenter)
            self.blocProgressBarText.setText("")

            self.progressBar = QProgressBar(self.blocProgressBar)
            self.progressBar.setProperty("value", 0)
            self.progressBar.setAlignment(Qt.AlignCenter)
            self.progressBar.setTextVisible(True)
            self.progressBar.setObjectName("progressBar")
            self.progressBar.setStyleSheet("QProgressBar {border: 2px solid grey; border-radius: 5px; text-align: center;}"
                                           "QProgressBar::chunk {background-color: #6C96C6; width: 20px;}")
            self.progressBar.setGeometry(70, 100, 260, 15)
            self.progressBar.setValue(0)

            self.myViewSubWindows = QScrollArea(self.mdi)
            self.myViewSubWindows.setObjectName("myViewSubWindows")
	    self.myViewSubWindows.setMinimumSize(300, 400)
	    self.myViewSubWindows.setMaximumSize(300, 400)
	    self.myViewSubWindows.setStyleSheet("QAbstractScrollArea {border: 2px solid grey; border-radius: 5px; text-align: center;background-color:#5D5D5D; padding: 4px ; color:rgb(255,255,255);}")
            self.listWindowsButtons = QListWidget(self.myViewSubWindows)
            self.listWindowsButtons.setGeometry(5, 5, 0, 0)
            self.listWindowsButtons.setObjectName("listWindowsButtons")
            self.listWindowsButtons.setIconSize(QSize(32, 32))
            self.listWindowsButtons.setStyleSheet("* { font-size:12px; background-color:#5D5D5D; padding: 4px ; color:rgb(255,255,255)}"
                                      "QListWidget::item:selected {color: #5D5D5D;background-color: rgb(255,255,255);  padding: 12px;}")           
            self.myViewSubWindows.setWidget(self.listWindowsButtons)
            self.myViewSubWindows.setFrameStyle(QFrame.NoFrame)
            self.myViewSubWindows.setFrameShadow(QFrame.Plain)
            self.myViewSubWindows.setWidget(QLabel().setPixmap(QPixmap.fromImage(QImage(getThemeIcon("noview.png")))))
            self.myViewSubWindows.setVerticalScrollBarPolicy(Qt.ScrollBarAlwaysOn)
            self.myViewSubWindows.setVisible(False)

            self.barInfo = QgsMessageBar(self.mdi)
            self.barInfo.setSizePolicy(QSizePolicy.Minimum, QSizePolicy.Fixed)

            self.listWindowsButtons.itemClicked.connect(self.changeActivateWindow)
            self.mdi.subWindowActivated.connect(self.fixeIndex)
            self.iface.mapCanvas().layersChanged.connect(self.majOpenFromLayers)
            try : self.iface.mapCanvas().currentLayerChanged.connect(self.majOpenFromLayers)
            except : pass
            self.iface.newProjectCreated.connect(self.majOpenFromLayers)

            self.xnewsession = MyPushButton(self)
            self.xnewsession.initPushButton(0, 0, 0, 0, "xnewsession", "", QApplication.translate("QSphere", "New session", None, QApplication.UnicodeUTF8), True, getThemeIcon("new.png"), 0, 0, False)
            self.xnewsession.setShortcut(QKeySequence("Ctrl+Shift+N"))
            self.xnewsession.clicked.connect(self.newSession)

            self.xopensession = MyPushButton(self)
            self.xopensession.initPushButton(0, 0, 0, 0, "xopensession", "", QApplication.translate("QSphere", "Load a session QSphere", None, QApplication.UnicodeUTF8), True, getThemeIcon("new.png"), 0, 0, False)
            self.xopensession.setShortcut(QKeySequence("Ctrl+Shift+F"))
            self.xopensession.clicked.connect(self.openSession)

            self.xsavesession = MyPushButton(self)
            self.xsavesession.initPushButton(0, 0, 0, 0, "xsavesession", "", QApplication.translate("QSphere", "Save current session", None, QApplication.UnicodeUTF8), True, getThemeIcon("save.png"), 0, 0, False)
            self.xsavesession.setShortcut(QKeySequence("Ctrl+Shift+W"))
            self.xsavesession.clicked.connect(self.saveSession)
            
            self.xnewfile = MyPushButton(self)
            self.xnewfile.initPushButton(0, 0, 0, 0, "xnewfile", "", QApplication.translate("QSphere", "New sheet", None, QApplication.UnicodeUTF8), True, getThemeIcon("new.png"), 0, 0, False)
            self.xnewfile.setShortcut(QKeySequence("Ctrl+N"))
            self.xnewfile.clicked.connect(self.newMetadataFile)
            
            self.xopenfile = MyPushButton(self)
            self.xopenfile.initPushButton(0, 0, 0, 0, "xopenfile", "", QApplication.translate("QSphere", "Load a metadata file", None, QApplication.UnicodeUTF8), True, getThemeIcon("open.png"), 0, 0, False)
            self.xopenfile.setShortcut(QKeySequence("Ctrl+F"))
            self.xopenfile.clicked.connect(self.openMetadataFile)

            self.xsavefile = MyPushButton(self)
            self.xsavefile.initPushButton(0, 0, 0, 0, "xsavefile", "", QApplication.translate("QSphere", "Save the current sheet QSphere", None, QApplication.UnicodeUTF8), True, getThemeIcon("save.png"), 0, 0, False)
            self.xsavefile.setShortcut(QKeySequence("Ctrl+W"))
            self.xsavefile.clicked.connect(self.saveMetadataFile)
            
            self.xsaveallfiles = MyPushButton(self)
            self.xsaveallfiles.initPushButton(0, 0, 0, 0, "xsaveallfiles", "", QApplication.translate("QSphere", "Save all files", None, QApplication.UnicodeUTF8), True, getThemeIcon("saveall.png"), 0, 0, False)
            self.xsaveallfiles.setShortcut(QKeySequence("Ctrl+A"))
            self.xsaveallfiles.clicked.connect(self.saveAllMetadataFile)
            
            self.refreshGroupWindowsButtons()
            self.majOpenFromLayers()
            self.changeStateButtons(None)
            self.translate(False)



        def translate(self, allItems):

            self.dicoLangs = self.parent.dicoLangs
            self.indexLang = self.parent.indexLang
                
            zElt0 = QApplication.translate("QSphere","project", None, QApplication.UnicodeUTF8).title()
            zElt1 = QApplication.translate("QSphere","metadata", None, QApplication.UnicodeUTF8)
            zElt2 = QApplication.translate("QSphere","session", None, QApplication.UnicodeUTF8)
            self.racWindowTitle = "%s :" % (zElt1.title())
            self.NamedFiltres = {"qsp": "%s QSphere (*.qsp)" % (zElt0), "xml" : "eXtensible Markup Language %s (*.xml)" %( zElt1), "qss" : "%s QSphere (*.qss)" % (zElt2)}                

            if not allItems : return

            nameTabs = ("Resources", "Edition", "Web and Services", "Windows", "Tools and Params")

            for i in range(self.tabWidget.count()):
                self.tabWidget.setTabText(i, QApplication.translate("QSphere", nameTabs[i], None, QApplication.UnicodeUTF8))
            
            listToTranslate = (self.listActionsTabFile, self.listActionsTabEdition, self.listActionsTabWebServices, self.listActionsTabWindows, self.listActionsTabOptions)
            for myList in listToTranslate :
                for item in myList :
                    if item[0] != "-" :
                       if item[0] == "~" :
                          Obj = self.findChild(QLabel, "Label%s_%s" % (item[1], item[2].split(".")[0]))
                          if Obj : Obj.setText("%s :" % QApplication.translate("QSphere", item[1], None, QApplication.UnicodeUTF8))
                       else :
                          Obj = self.findChild(MyPushButton, item[0])
                          if Obj :
                                  zText = QApplication.translate("QSphere",item[1], None, QApplication.UnicodeUTF8)
                                  if item[0] == "newSession" : zText = "%s%s" % (zText, "Ctrl+Shift+N".rjust(60-len(zText)))
                                  if item[0] == "openSession" : zText = "%s%s" % (zText, "Ctrl+Shift+F".rjust(60-len(zText)))
                                  if item[0] == "saveSession" : zText = "%s%s" % (zText, "Ctrl+Shift+W".rjust(60-len(zText)))
                                  if item[0] == "newFile" : zText = "%s%s" % (zText, "Ctrl+N".rjust(60-len(zText)))
                                  if item[0] == "openFile" : zText = "%s%s" % (zText, "Ctrl+F".rjust(60-len(zText)))
                                  if item[0] == "saveFile" : zText = "%s%s" % (zText, "Ctrl+W".rjust(60-len(zText)))
                                  if item[0] == "saveAll" : zText = "%s%s" % (zText, "Ctrl+A".rjust(60-len(zText)))
                                  Obj.setToolTip("%s" % QApplication.translate("QSphere", zText, None, QApplication.UnicodeUTF8))

            Obj = self.findChild(MyPushButton, "openSession")    
            if Obj : self.buildContextMnuSession(Obj)

            Obj = self.findChild(MyPushButton, "saveSessionAs")
            if Obj : self.contextMnuSaveSessionActions(Obj)

            Obj = self.findChild(MyPushButton, "openFile")
            if Obj : self.buildContextMnuFile(Obj)

            Obj = self.findChild(MyPushButton, "saveFileAs")
            if Obj : self.contextMnuSaveActions(Obj)

            Obj = self.findChild(MyPushButton, "navigatorWeb")
            if Obj : self.buildContextMnuNavigatorWeb(Obj)


        def fixeMDDWindow(self):
            dialog = None     
            if self.mdi.subWindowList(QMdiArea.CreationOrder)!= [] : 
               dialog = self.mdi.activeSubWindow()
               if not dialog : 
                   dialog = self.mdi.subWindowList(QMdiArea.CreationOrder)[0]
                   self.mdi.setActiveSubWindow(dialog)
               if dialog != None :
                  for child in dialog.children():
                      if type(child)==DialogMetadata : return dialog, child
            return dialog, None    

        def openHelp(self): makeHelp(self)
        def openHelpINSPIRE(self):
            try : webbrowseropen("http://cnig.gouv.fr/?page_id=2916")
            except : pass

        def loadNavigator(self, fileName):
            if self.webNavigator == None :
               self.webNavigator = LoadDialogViewer(self, self.iface, fileName, False, [], self.langueTR, self, None, "navigatorweb.png", False)
               self.windowsNoMDIChilds.append(self.webNavigator)
            else :
               self.webNavigator.addNewTab()
               self.webNavigator.endLoadTheFile(fileName)
            self.webNavigator.show()

        def changeMetadataMode(self):
            dialog, child = self.fixeMDDWindow()
            if child :               
               idbutton = self.changeMode.id(self.changeMode.checkedButton())
               child.doChangeMode(idbutton)
               child.histoIDChangeMode = idbutton


        def reloadWindow(self):
            dialog, child = self.fixeMDDWindow()
            if child :               
                      self.blocProgressBarButton.setIcon(self.sender().icon())
                      self.blocProgressBarText.setText("%s :\n %s" % (self.sender().toolTip(), child.windowTitle()))
                      self.blocProgressBar.setVisible(True)     
                      child.ReLoadMetadata()
                      self.blocProgressBar.setVisible(False)

        def majOpenFromLayers(self):
            but = self.findChild(MyPushButton, "openFromLayers")
            if but : but.setEnabled(self.iface.mapCanvas().layerCount() > 0)
                
        def fixeIndex(self):
            dialog, child = self.fixeMDDWindow()
            if child :
               index = self.synchroIconTab(child, False, -1)
               self.nb_subwindows.setText("%s" % (len(self.mdi.subWindowList(QMdiArea.CreationOrder))))
               self.nb_subwindows.setAlignment(Qt.AlignCenter)
               self.nb_window_childs.setText("%s" % (len(child.childswindows)))
               self.nb_window_childs.setAlignment(Qt.AlignCenter)
               self.fixeModeDialog(child.histoIDChangeMode, index)

               if type(self.sender()) == QMdiArea :
                       for tuples in self.listSubWindows :
                           if dialog in tuples :
                              self.listWindowsButtons.setCurrentItem(tuples[0])
                              break

               self.changeStateButtons(child)

               if child.indexLang != self.MainPlugin.indexLang : child.changeLang()

        def changeStateButtons(self, child):
            controlButtonsEnabled = {"saveSession" : 0, "saveFile" : 3, "saveAll" : 1, "reloadWindow" : 3, "wizForm" : 1, "wizEditorXML" :  3,
                                     "wizContacts" : 1, "addCSWTButton" : 2, "editCSWTButton" : 2, "delCSWTButton" : 2,
                                     "tileWindows" : 1, "cascadeWindows" : 1, "batteriesWindows" : 1, "closeWindow" : 1, "closeAllWindows" : 1}

            borne = self.listWindowsButtons.count()
            for key in controlButtonsEnabled :
                but = self.findChild(MyPushButton, key)         
                if but :
                      if controlButtonsEnabled[key] == 0: but.setEnabled(self.windowTitle() != self.racTitle)  
                      elif controlButtonsEnabled[key] == 1: but.setEnabled(child != None and borne > 0)
                      elif controlButtonsEnabled[key] == 2: but.setEnabled(child != None and borne > 0 and self._connexionCSWT != None)
                      else :
                           if child : but.setEnabled(child.windowTitle() != child.racWindowTitle and child.windowTitle().find("~")== -1)
                           else : but.setEnabled(False)

            self.tabWidget.setTabEnabled(1, (child != None and borne > 0))
                           

        def fixeModeDialog(self, idbutton, indexpostab):
            nameObj = ("formMode", "xmlMode", "htmlMode")[idbutton]
            Obj = self.findChild(MyPushButton, nameObj)
            if Obj : Obj.setChecked(True)
            nameObj = self.listNameActionsTabEdition[indexpostab]
            Obj = self.findChild(MyPushButton, nameObj)
            if Obj : Obj.setChecked(True)
            
                           
        def fixeMetaTab(self):
            if self.mdi.subWindowList(QMdiArea.CreationOrder)!= [] :
               dialog = self.mdi.activeSubWindow()
               index = self.listNameActionsTabEdition.index(self.sender().objectName())
               for child in dialog.children():
                   if type(child)==DialogMetadata :
                          index = self.synchroIconTab(child, True, index)
                          break
                        
        def synchroIconTab(self, child, fixeIndex, index):
             if not fixeIndex : index = child.tabWidget.currentIndex()
             else : child.tabWidget.setCurrentIndex(index)
             myDefPathIcon = getThemeIcon(self.listActionsTabEdition[index+1][2])
             carIcon = QImage(myDefPathIcon)
             child.currentTab.setPixmap(QPixmap.fromImage(carIcon).scaled(48, 48))
             if type(self.sender()) == MyPushButton : child.currentTab.setToolTip(self.sender().toolTip())
             return index   

        def printMetaTab(self):
            dialog, child = self.fixeMDDWindow()
            if child : child.PrintMetadata()
            

        #========================
        # BLOC ACTIONS FILES MDD
        #========================                
        def initMetadataDialog(self):
            try : icon = self.sender().icon()
            except :
               try : icon = self.sender().currentItem().icon()
               except : icon = QIcon(getThemeIcon("unknow.png"))
            self.blocProgressBarButton.setIcon(icon)
            
            try : self.blocProgressBarText.setText("%s" % self.sender().toolTip())
            except :
                try : self.blocProgressBarText.setText("%s" % self.sender().mainFrame().url().toString())
                except : self.blocProgressBarText.setText("")
            self.blocProgressBar.setVisible(True)
                
            dialog =  DialogMetadata(self.iface, self.langue, self.languageIndex, self.langs, self.languesDico, \
                            self.langueTR, self.formats, self.listCodecs, self.listTemporalSystem, \
                            self.listTypeRessources, self.listCountries, self.indexCountry, self.listCountriesCode, self.localeFullName, self)
            dialog.setWindowTitle("%s ~ %s" % (dialog.windowTitle(), self.counterNew))    
            mdiWindow = mySubWindow(self, dialog, QIcon(getThemeIcon("openmeta.png")))   
            self.mdi.addSubWindow(mdiWindow)
            self.mdi.setActiveSubWindow(mdiWindow)

            self.blocProgressBar.setVisible(False)
            return dialog, mdiWindow

        def majSuperviseur(self, mdiWindow):
            myToolTip = mdiWindow.windowTitle()
            myName = basename(myToolTip)
            icon = getThemeIcon("openmeta.png") 
            Obj =  QListWidgetItem(QIcon(icon), myName, self.listWindowsButtons)
            Obj.setToolTip(myToolTip)
            self.listSubWindows.append((Obj, mdiWindow))
            self.listWindowsButtons.setCurrentRow(self.listWindowsButtons.count()-1)
            self.refreshGroupWindowsButtons()
            dialog, child = self.fixeMDDWindow()
            self.changeStateButtons(child)


        def newSession(self):
            self.currentSession = None
            self.setWindowTitle(self.racTitle)
            self.butsavesession.setEnabled(self.currentSession != None)
            self.closeAllMySubWindows()

        def openLastSession(self) :
            icon = self.sender().icon()
            text = QApplication.translate("QSphere","Load a session file", None, QApplication.UnicodeUTF8)
            if self.sender().text()!= "" :  self.loadTheSession(self.sender().text(), icon, text)

        def openLastFile(self):
            icon = self.sender().icon()
            text = QApplication.translate("QSphere","Load a session file", None, QApplication.UnicodeUTF8)
            fileName = self.sender().text()
            if exists(fileName) :
               self.loadMDDFile(fileName, False, -1)     
               self.InitDir = dirname(fileName)

        def openSession(self):
            extension = "qss"
            icon = self.sender().icon()
           
            text = QApplication.translate("QSphere","Load a session file", None, QApplication.UnicodeUTF8)
            MyFileDialog = QFileDialog(self, text)
            MyFileDialog.setNameFilters([items[1] for items in self.NamedFiltres.items() if items[0]==extension]) 
            MyFileDialog.selectNameFilter(self.NamedFiltres[extension])
            MyFileDialog.setDefaultSuffix(extension)
            MyFileDialog.setViewMode(QFileDialog.Detail)
            MyFileDialog.setDirectory(self.InitDir)
            MyFileDialog.setFileMode(QFileDialog.ExistingFile)  
            MyFileDialog.setAcceptMode(QFileDialog.AcceptOpen)

            FixeLabelsFileDialog(self, MyFileDialog, 0, True)
                
            if MyFileDialog.exec_():
               fileName = "%s" % (MyFileDialog.selectedFiles()[0])
               if fileName!="" :
                  self.loadTheSession(fileName, icon, text)
                  self.ajustListInfos(fileName, self.listSessions)


        def loadTheSession(self, fileName, icon, text):       
                  self.counterNew = 0
                  self.currentSession = fileName
                  self.setWindowTitle("%s : %s" % (self.racTitle, fileName))
                  self.InitDir = dirname(fileName)

                  self.blocProgressBarButton.setIcon(icon)
                  self.blocProgressBarText.setText("%s :\n %s" % (text,fileName))
                  self.blocProgressBar.setVisible(True)  
                  
                  self.closeAllMySubWindows()
		  self.tree = ET.parse(fileName) 
		  self.root = self.tree.getroot()

                  formMainUI = self.root.findall("formMainUI")
                  for node in formMainUI:
                          try : 
                                  test = eval(node.find("modeWindow").text)
                                  self.myViewSubWindows.setVisible(test)
                                  Obj = self.findChild(MyPushButton, "modeWindow")
                                  if Obj : Obj.setIcon(QIcon(getThemeIcon("windowmdiarea_%s.png".lower() % self.myViewSubWindows.isVisible())))
                          except : pass
                  
                  subWindows = self.root.findall("subWindow")
                  dialogFocused = None
                  counterTemp = 0

                  for node in subWindows :
                      dicoSubWindow = self.parse_subWindow(node)
                      source = dicoSubWindow["source"]
                      dialog, mdiWindow = self.loadMDDFile(source, False, -1) if source !="" and source else self.newMetadataFile()

                      if not source :
                         dataQSP = dicoSubWindow["projecttemp"]

                         from StringIO import StringIO as bufferData
                         from xmlISOparser import xmlISOparser
                         buffer = bufferData(dataQSP) 
                         myISO = xmlISOparser(None, buffer , 'MEDDE', 'fr') 
                         myISO.createISOdataStructure(True)

                         try :       
                            if myISO.getTagDictionnary() :
                              dialog.ReloadDataFromXML(myISO, None, 0)
                              dialog.setWindowTitle("%s ~ %s" % (dialog.windowTitle(), counterTemp))
                              counterTemp+= 1
                         except :
                            try : result = dialog.GetDataFromFluxQSP(dataQSP)
                            except : pass
                      
                      if dialog :
                         posx = dicoSubWindow["posx"] 
                         posy = dicoSubWindow["posy"] 
                         width = dicoSubWindow["width"]
                         height = dicoSubWindow["height"]
                         try :mdiWindow.setGeometry(int(posx), int(posy), int(width), int(height))
                         except : pass
                         if dicoSubWindow["activate"]=="True" : dialogFocused = mdiWindow

                         try : dialog.tabWidget.setCurrentIndex(int(dicoSubWindow["tabwidgetcurrentindex"]))
                         except : pass

                         try : 
                               idbutton = int(dicoSubWindow["histoIDChangeMode"])  
                               dialog.doChangeMode(idbutton)
                               dialog.histoIDChangeMode = idbutton

                         except : pass                        
                  if dialogFocused :       
                          self.mdi.setActiveSubWindow(dialogFocused)      
                          dialogFocused.setFocus()
                  self.butsavesession.setEnabled(self.currentSession != None)
                  self.blocProgressBar.setVisible(False)
                  self.fixeIndex()
                  
        def parse_subWindow(self, node):
            dicoSubWindow = {}
            keys = ("title", "source", "posx", "posy", "width", "height", "activate", "projecttemp", "tabwidgetcurrentindex", "histoIDChangeMode")
            for key in keys :
                try : dicoSubWindow[key] = node.find(key).text
                except :
                       if key in ("title", "source", "activate", "projecttemp") : dicoSubWindow[key] = ""
                       elif key in ("posx", "posy", "width", "height", "tabwidgetcurrentindex", "histoIDChangeMode"): dicoSubWindow[key] = 0
                       elif key == "activate" : dicoSubWindow[key] = "False" 
            return dicoSubWindow

               
        def saveSession(self) :
            extension = "qss"
            if self.sender() == self.SaveSessionAsButton : zTitle = QApplication.translate("QSphere","Save session as file", None, QApplication.UnicodeUTF8)
            elif self.sender() == self.SaveSessionCopyAsButton : zTitle = QApplication.translate("QSphere","Save session as copy file", None, QApplication.UnicodeUTF8)
            else : zTitle = QApplication.translate("QSphere","Save session file", None, QApplication.UnicodeUTF8)        

            fileName = ""        
            if self.sender() in (self.SaveSessionAsButton, self.SaveSessionCopyAsButton) or self.currentSession == None :
                    MyFileDialog = QFileDialog(self, zTitle)
                    MyFileDialog.setNameFilters([items[1] for items in self.NamedFiltres.items() if items[0]==extension]) 
                    MyFileDialog.selectNameFilter(self.NamedFiltres[extension])
                    MyFileDialog.setDefaultSuffix(extension)
                    MyFileDialog.setViewMode(QFileDialog.Detail)
                    MyFileDialog.setDirectory(self.InitDir)
                    MyFileDialog.setAcceptMode(QFileDialog.AcceptSave)

                    FixeLabelsFileDialog(self, MyFileDialog, 1, True)
                        
                    if MyFileDialog.exec_():
                       fileName = "%s" % (MyFileDialog.selectedFiles()[0])
            else :
                    fileName = self.currentSession    
                                      
            if fileName!="" :
               self.doSaveSession(fileName)
               
               if self.sender() != self.SaveSessionCopyAsButton :
                  self.ajustListInfos(fileName, self.listSessions)
                  self.InitDir = dirname(fileName)
                  self.currentSession = fileName
                  self.setWindowTitle("%s : %s" % (self.racTitle, fileName))

               zTitle = QApplication.translate("QSphere", "Information" , None, QApplication.UnicodeUTF8) 
               zMsg = QApplication.translate("QSphere","The session's file was saved as", None, QApplication.UnicodeUTF8)
               SendMessage(self, zTitle , "%s :<br><u><i>%s</i></ul>" % (zMsg, fileName), QgsMessageBar.INFO, self.duration_info)


        def doSaveSession(self, fileName):                
            zLOG = open(fileName, "w")
            WriteInLOG(zLOG, '<?xml version="1.0" encoding="UTF-8"?>\n')
            WriteInLOG(zLOG, '<gmd:MD_Metadata xsi:schemaLocation="http://www.isotc211.org/2005/gmd http://schemas.opengis.net/iso/19139/20060504/gmd/gmd.xsd" xmlns:gmd="http://www.isotc211.org/2005/gmd" xmlns:gco="http://www.isotc211.org/2005/gco" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:gml="http://www.opengis.net/gml" xmlns:xlink="http://www.w3.org/1999/xlink">\n')
            WriteInLOG(zLOG, '\t<gmd:qspheresession>\n')
            WriteInLOG(zLOG, '\t\t<gco:file>\n')
            WriteInLOG(zLOG, '\t\t\t<gco:CharacterString>%s</gco:CharacterString>\n' % (fileName))
            WriteInLOG(zLOG, '\t\t</gco:file>\n')
            WriteInLOG(zLOG, '\t\t<gmd:language>\n')
            WriteInLOG(zLOG, '\t\t\t<gmd:LanguageCode codeList="http://www.loc.gov/standards/iso639-1/" codeListValue="%s">%s</gmd:LanguageCode>\n' % (self.langueTR, self.langueTR))
            WriteInLOG(zLOG, '\t\t</gmd:language>\n')
            MakeDateStampXML(self, 1, False,  None)
            WriteInLOG(zLOG, '\t</gmd:qspheresession>\n')
            
            WriteInLOG(zLOG, '\t<formMainUI name="%s">\n' % (self.objectName()))
            WriteInLOG(zLOG, '\t\t<postab>%s</postab>\n' % self.tabWidget.currentIndex())
            WriteInLOG(zLOG, '\t\t<modeWindow>%s</modeWindow>\n' % (self.myViewSubWindows.isVisible()))
            WriteInLOG(zLOG, '\t</formMainUI>\n')
            
            counter=0

            for subWindow in self.mdi.subWindowList(QMdiArea.CreationOrder):
                WriteInLOG(zLOG, '\t<subWindow>\n')
                WriteInLOG(zLOG, '\t\t<id>%s</id>\n' % counter)           
                for child in subWindow.children():
                      if type(child)==DialogMetadata :
                         fileName = ""     
                         if child.windowTitle() != child.racWindowTitle and child.windowTitle().find("~")== -1 : fileName = child.windowTitle().replace(child.racWindowTitle, "")
                         WriteInLOG(zLOG, '\t\t<source>%s</source>\n' % fileName.encode("utf-8"))
                         if fileName == "" :
                            WriteInLOG(zLOG, '\t\t<projecttemp><![CDATA[%s]]></projecttemp>\n' % ExportToXML(child, None))      
                         else : WriteInLOG(zLOG, '\t\t<projecttemp></projecttemp>\n')
                         WriteInLOG(zLOG, '\t\t<tabwidgetcurrentindex>%s</tabwidgetcurrentindex>\n' % child.tabWidget.currentIndex())
                         WriteInLOG(zLOG, '\t\t<histoIDChangeMode>%s</histoIDChangeMode>\n' % child.histoIDChangeMode)
                         break     
                WriteInLOG(zLOG, '\t\t<posx>%s</posx>\n' % subWindow.x())
                WriteInLOG(zLOG, '\t\t<posy>%s</posy>\n' % subWindow.y())
                WriteInLOG(zLOG, '\t\t<width>%s</width>\n' % subWindow.width())
                WriteInLOG(zLOG, '\t\t<height>%s</height>\n' % subWindow.height())
                WriteInLOG(zLOG, '\t\t<activate>%s</activate>\n' % (subWindow == self.mdi.activeSubWindow()))    
                WriteInLOG(zLOG, '\t</subWindow>\n')
                counter+= 1
                
            WriteInLOG(zLOG, "</gmd:MD_Metadata>\n" )
            CloseLOG(zLOG)
            

    
        def newMetadataFile(self):
            dialog, mdiWindow = self.initMetadataDialog()
            dialog.show()
            self.counterNew+= 1
            self.majSuperviseur(mdiWindow)
            self.tabWidget.setCurrentIndex(1)
            return dialog, mdiWindow

        def openMetadataFileForLayers(self): 
            dialog = LoadFromLayersDialog(self.iface, self)
            MakeWindowIcon(dialog, "openfromlayers.png")
            if dialog.exec_():
               for elt in dialog.qgisLayersList():
                   infos = elt.split("|")    
                   fileName = infos[0]
                   layerid  = infos[1]
                   if fileName!="" :
                      self.loadMDDFile(fileName, True, layerid)     
                      self.InitDir = dirname(fileName)
                      

        def openMetadataFile(self):
                extension = self.extension 
                zTitle = QApplication.translate("QSphere","Load a metadata file", None, QApplication.UnicodeUTF8)
                MyFileDialog = QFileDialog(self, zTitle)
                MyFileDialog.setNameFilters([items[1] for items in self.NamedFiltres.items() if items[0]!="qss"])  
                MyFileDialog.selectNameFilter(self.NamedFiltres[extension])
                MyFileDialog.setDefaultSuffix(extension)
                MyFileDialog.setViewMode(QFileDialog.Detail)
                MyFileDialog.setDirectory(self.InitDir)
                MyFileDialog.setFileMode(QFileDialog.ExistingFiles)  
                MyFileDialog.setAcceptMode(QFileDialog.AcceptOpen)

                FixeLabelsFileDialog(self, MyFileDialog, 0, True)
                
                if MyFileDialog.exec_():
                    if  MyFileDialog.selectedFiles() != [] :   
                        for fileName in MyFileDialog.selectedFiles() :
                            if fileName!="" :
                               self.loadMDDFile(fileName, False, -1)
                               self.InitDir = dirname(fileName)
                               self.ajustListInfos(fileName, self.listMDDFiles)

        def loadMDDFile(self, fileName, fromLayer, layerid):
                dialog, mdiWindow = self.initMetadataDialog()
                self.fixeModeDialog(0, 0)
                extension = GetExtension(fileName)
                self.extension = extension.replace(".","")
                try : icon = self.sender().icon()
                except :
                     try : icon = self.sender().currentItem().icon()
                     except : icon = QIcon(getThemeIcon("unknow.png"))
                self.blocProgressBarButton.setIcon(icon)
                self.blocProgressBarText.setText("%s :\n %s" % (self.sender().toolTip(), fileName))
                self.blocProgressBar.setVisible(True)
                if extension == ".qsp" : dialog.GetDataFromQSP(fileName)
                elif extension == ".xml" : dialog.GetDataFromXML(fileName)
                elif extension == ".xqsp" : dialog.GetDataFomXQSP(fileName)

                if fromLayer :
                   IndexLayer = GetLayerCombo(dialog, dialog.ComboLayers, layerid)
                   dialog.ComboLayers.setCurrentIndex(IndexLayer)
                   
                   if dialog.ComboLayers.currentIndex() > 0 : 
                      dialog.IsVisibleRestoreDico(True)
                      dialog.GetMetaData()
                      if dialog.autoComment and extension == ".qsp" : dialog.RestoreComments()
                      dialog.IsVisibleRestoreDico(extension == ".qsp")
                
                self.blocProgressBar.setVisible(False)
                dialog.show()
                self.majSuperviseur(mdiWindow)
                self.tabWidget.setCurrentIndex(1)
                return dialog, mdiWindow

        def DownloadMetadata(self):
                zicon, zurl = "navigatorweb.png", "#blank"
                if self.sender().objectName() == "openHelp" :
                    zLang = "_%s" % (self.langueTR) if self.langueTR!= "en" else ""
                    zicon, zurl = "help.png", getThemeIcon("ressources/html/help/index%s.html" % (zLang))
                else :
                    zurl = self.sender().text() if self.sender().objectName()!= "mActionNavigator" else ""
                self.loadNavigator(zurl)

        def saveAllMetadataFile(self):
            listSubWindows = self.mdi.subWindowList(QMdiArea.CreationOrder)    
            if listSubWindows!= [] :
               for subWindow in listSubWindows :     
                       for child in subWindow.children():
                           if type(child)==DialogMetadata :
                                  if child.windowTitle() != child.racWindowTitle and child.windowTitle().find("~")== -1 :                
                                          child.SaveData()
                                          break
                                
        def saveMetadataFile(self):
            dialog, child = self.fixeMDDWindow()
            if child : child.SaveData()
                          
               
        def saveAsMetadataFile(self):
            index = 2 if self.sender().objectName() == "SaveCopyAsButton" else 1
            dialog, child = self.fixeMDDWindow()
            if child :
               child.InitDir = self.InitDir  
               child.SaveDataAs(index)
            

        def saveXmlCatFile(self):
            dialog, child = self.fixeMDDWindow()
            if child : child.SaveXmlCat()

        #===============================
        # BLOC ACTIONS EDITION - WIZARDS
        #===============================
        def openFormWizardMode(self):
            dialog, child = self.fixeMDDWindow()
            if child : child.OpenFormTable(child.focusWidget())
                      
        def openXMLEditorWizardMode(self):
            dialog, child = self.fixeMDDWindow()
            if child :
               if child.windowTitle() != child.racWindowTitle and child.windowTitle().find("~")== -1 :                
                    fileName = child.windowTitle().replace(child.racWindowTitle, "")
                    textension = splitext(fileName)
                    extension = textension[len(textension)-1].lower()
                    if extension == ".xml" :
                       d = xmlEditor(child, fileName)
                       MakeWindowIcon(d, "wizeditxml.png")
                       d.setWindowTitle("%s : %s" % (d.racTitle, fileName))
                       d.LoadFile()
                       d.exec_()
                    else :
                       dataXML = ExportToXML(child, None)
                       self.getReporting(dataXML.decode("utf-8"), False, "wizeditxml.png", True)

        def openContactsWizardMode(self):
            dialog, child = self.fixeMDDWindow()
            if child :
               orIndex = child.tabWidget.currentIndex()
               child.tabWidget.setCurrentIndex(6)
               nameObj = self.listNameActionsTabEdition[6]
               Obj = self.findChild(MyPushButton, nameObj)
               if Obj : Obj.setChecked(True)                           
               d = DialogContacts(child, True)
               d.exec_()
               child.tabWidget.setCurrentIndex(orIndex)

        #========================
        # BLOC ACTIONS WINDOWS
        #========================             
        def tileWindows(self):
            dialog, child = self.fixeMDDWindow()    
            self.mdi.tileSubWindows()    
            self.mdi.setActiveSubWindow(dialog)
            dialog.setFocus()
            
        def cascadeWindows(self):
            dialog, child = self.fixeMDDWindow()        
            self.mdi.cascadeSubWindows()
            self.mdi.setActiveSubWindow(dialog)
            dialog.setFocus()

        def batteriesWindows(self):
            subWindows = self.mdi.subWindowList(QMdiArea.CreationOrder)
            N = len(subWindows)
            if N == 0 : return    

            from math import pi, sin ,cos, sqrt    
            self.cascadeWindows()
            dialog, child = self.fixeMDDWindow()

            centerX = subWindows[0].width()
            dialog.setGeometry(0, 0, dialog.width(), dialog.height())
            elt = 0
            for itheta in range(N):
                if subWindows[itheta]!=dialog :
                   subWindows[itheta].setGeometry(centerX, 100*elt, subWindows[itheta].width(), subWindows[itheta].height())
                   elt+= 1
                
            self.mdi.setActiveSubWindow(dialog)
            dialog.setFocus()
            self.mdi.horizontalScrollBar().setValue(0)
            self.mdi.verticalScrollBar().setValue(0)
                

        def closeAllMySubWindows(self):
            self.counterNew = 0
            for i in range(len(self.listSubWindows)-1, -1, -1) :
                tuples = self.listSubWindows[i]    
                mdiWindow = tuples[1]
                for child in mdiWindow.children():
                      if type(child)==DialogMetadata : child.killWindows(); break;
                self.mdi.removeSubWindow(mdiWindow)
                self.listWindowsButtons.takeItem(self.listWindowsButtons.row(tuples[0]))
                self.listSubWindows.remove(tuples)
                
            self.nb_subwindows.setText("%s" % (len(self.mdi.subWindowList(QMdiArea.CreationOrder))))    
            self.listSubWindows = []
            self.listWindowsButtons.clear()
            self.refreshGroupWindowsButtons()
            self.changeStateButtons(None)
            

        def refreshGroupWindowsButtons(self):
            borne = self.listWindowsButtons.count()    
            if borne == 0 : self.listWindowsButtons.setGeometry(self.listWindowsButtons.x(), self.listWindowsButtons.y(), 0, 0)
            else : self.listWindowsButtons.setGeometry(self.listWindowsButtons.x(), self.listWindowsButtons.y(), 300, 35*borne+ (10*(min(borne, 1))))
           
            
        def closeCurrentWindow(self):
            dialog, child = self.fixeMDDWindow()
            if child :
               child.killWindows()
               for tuples in self.listSubWindows :
                   if dialog in tuples :
                      self.listWindowsButtons.takeItem(self.listWindowsButtons.row(tuples[0]))
                      self.mdi.removeSubWindow(tuples[1])
                      self.listSubWindows.remove(tuples)
                      
               self.nb_subwindows.setText("%s" % (len(self.mdi.subWindowList(QMdiArea.CreationOrder))))
               self.refreshGroupWindowsButtons()
               dialog, child = self.fixeMDDWindow()
               self.changeStateButtons(child)
               
               

        def changeViewWindowsButtons(self):
             self.myViewSubWindows.setVisible(not self.myViewSubWindows.isVisible())
             Obj = self.findChild(MyPushButton, "modeWindow")
             if Obj : Obj.setIcon(QIcon(getThemeIcon("windowmdiarea_%s.png".lower() % self.myViewSubWindows.isVisible())))

        def changeActivateWindow(self): #, index
            item = self.listWindowsButtons.currentItem()
            for tuples in self.listSubWindows :
                if tuples[0] == item :
                    activeSubWindow = self.mdi.currentSubWindow()
                    newposition = (activeSubWindow.x(), activeSubWindow.y(), activeSubWindow.width(), activeSubWindow.height())
                    newActiveSubWindow = tuples[1] 
                    oldposition = (newActiveSubWindow.x(), newActiveSubWindow.y(), newActiveSubWindow.width(), newActiveSubWindow.height())

                    activeSubWindow.setGeometry(oldposition[0], oldposition[1], oldposition[2], oldposition[3])    
                    newActiveSubWindow.setGeometry(newposition[0], newposition[1], newposition[2], newposition[3])    
                    
                    self.mdi.setActiveSubWindow(newActiveSubWindow)
                    break

            
        #========================
        # BLOC ACTIONS WEB
        #========================                       
        def openLustreWizard(self):
            if self.dialogLUSTRE == None :    
                self.dialogLUSTRE = LusTREDialog(self, "http://linkeddata.ge.imati.cnr.it/", False)
                self.windowsNoMDIChilds.append(self.dialogLUSTRE)
            self.dialogLUSTRE.show()


        #CSW-T BLOC FUNTIONS
        def setEnabledCSWTActions(self):
                zCond = True if (self._connexionCSWT != None and self._connexionCSWT.connected()) else False
                for item in self.listNameActionsTabWebServices :
                    Obj = self.findChild(MyPushButton, item)
                    if Obj :
                       Obj.setEnabled(not zCond) if item == "authCSWTButton" else Obj.setEnabled(zCond)
                       dialog, child = self.fixeMDDWindow()
                       self.changeStateButtons(child)
                       
        def authCSWT(self):
                nameCSWT = getSelConnexion(self)
                mySettings = QSettings()
                serverCSWT = (mySettings.value("/qsphere/connections/%s/host" % (nameCSWT)))
                authServerCSWT = (mySettings.value("/qsphere/connections/%s/authhost" % (nameCSWT)))
                UserCSWT = (mySettings.value("/qsphere/connections/%s/username" % (nameCSWT)))
                pwdUserCSWT = (mySettings.value("/qsphere/connections/%s/password" % (nameCSWT)))

                dialog = AuthCSWTDialog(UserCSWT, pwdUserCSWT, serverCSWT)
                MakeWindowIcon(dialog, "connect.png")
                if dialog.exec_():
                   self.startMovie()

                   self._connexionCSWT = cswtConnection(self, nameCSWT, serverCSWT, authServerCSWT)
                   responseCSWT = self._connexionCSWT.initSession(dialog.authInfos()[0], dialog.authInfos()[1], True) 
                   self.setEnabledCSWTActions()

                   Obj = self.findChild(MyPushButton, "authCSWTButton")

                   if Obj :     
                           if not Obj.isEnabled():
                              zTitle = QApplication.translate("QSphere", "Information" , None, QApplication.UnicodeUTF8) 
                              zMsg = QApplication.translate("QSphere","Connected with the server", None, QApplication.UnicodeUTF8)
                              zPicto = QgsMessageBar.INFO
                              zDuration = self.duration_info
                           else :
                              zTitle = QApplication.translate("QSphere", "Warning" , None, QApplication.UnicodeUTF8) 
                              zMsg = QApplication.translate("QSphere","Error", None, QApplication.UnicodeUTF8)
                              zPicto = QgsMessageBar.WARNING
                              zDuration = self.duration_warning
                  
                           SendMessage(self, zTitle, "%s" % (zMsg), zPicto, zDuration)
                           
                   if responseCSWT == None : self.getAllReporting()

                   self.stopMovie()


        def disconnectCSWT(self):
                if self._connexionCSWT != None :
                    if self._connexionCSWT.connected() :
                       self._connexionCSWT.closeSession()
                       self.setEnabledCSWTActions()
                       self._connexionCSWT = None
                       
                       Obj = self.findChild(MyPushButton, "authCSWTButton")
                       if Obj :
                               if Obj.isEnabled():
                                  zTitle = QApplication.translate("QSphere", "Information" , None, QApplication.UnicodeUTF8) 
                                  zMsg = QApplication.translate("QSphere","Disconnected with the server", None, QApplication.UnicodeUTF8)
                                  zPicto = QgsMessageBar.INFO
                                  zDuration = self.duration_info
                               else :
                                  zTitle = QApplication.translate("QSphere", "Warning" , None, QApplication.UnicodeUTF8) 
                                  zMsg = QApplication.translate("QSphere","Error", None, QApplication.UnicodeUTF8)
                                  zPicto = QgsMessageBar.WARNING
                                  zDuration = self.duration_warning
                      
                               SendMessage(self, zTitle, "%s" % (zMsg), zPicto, zDuration)   

        def getAllReporting(self):
                if self._connexionCSWT != None : self.getReporting(self._connexionCSWT.allreporting(), False, "editxml.png", True)
        
        def getReporting(self, responseCSWT, isReporting, iconpath, isModal):
                d = xmlEditor(self, "")
                MakeWindowIcon(d, iconpath)
                d.setWindowTitle("%s : %s" % (d.racTitle, ""))
                d.show() if isModal else d.exec_()

                if isReporting :
                    try : charset = (responseCSWT.headers["Content-Type"].split("=")[1].strip())
                    except : charset = "utf-8"  
                    try : d.editor.setText("%s%s" % (self._connexionCSWT.reporting(), responseCSWT.read().decode(charset)))
                    except : d.editor.setText("%s%s" % (self._connexionCSWT.reporting(), responseCSWT))
                else :
                    d.editor.setText(responseCSWT)
                    
                d.checkProperties()
                d.BadMyISO()


        def recordsInCSWT(self): self.actionsInCSWT(6)                
        def insertInCSWT(self): self.actionsInCSWT(1)
        def updateInCSWT(self): self.actionsInCSWT(2)
        def deleteInCSWT(self): self.actionsInCSWT(-1)

        def actionsInCSWT(self, ucase):
                dialog, child = self.fixeMDDWindow()
                if child or ucase == 6 :        
                        self.startMovie()
                        if ucase in (1,2) :
                            #if child.windowTitle()!=child.racWindowTitle :
                            if child.windowTitle() != child.racWindowTitle and child.windowTitle().find("~")== -1 :        
                                child.SaveData()
                                fileName = child.windowTitle().replace(child.racWindowTitle, "")
                                extension = GetExtension(fileName)
                                if extension == ".xml" :
                                    f = open(fileName, "r")
                                    dataXML = f.read()
                                    f.close()
                                elif extension == ".qsp" : dataXML = ExportToXML(child, None)
                            else :
                                dataXML = ExportToXML(child, None)
                                
                            pos = dataXML.find("<gmd:MD_Metadata")
                            dataXML = dataXML[pos:]
                            
                        elif ucase == -1 :
                            zObj = child.findChild(QLineEdit, "identificator")
                            if zObj : dataXML = zObj.text()
                            else : return
                        elif ucase == 6: dataXML = None    
                        else : return

                        if self._connexionCSWT != None :
                           responseCSWT = self._connexionCSWT.metadata_csw_publication(ucase, dataXML, True) 
                           if ucase != 6 : self._connexionCSWT.analyseResponse(ucase, responseCSWT)

                           if self.reportingCSWT : self.getReporting(responseCSWT, True, "editxml.png", True)

                        self.stopMovie()

        #END BLOC CSWT

        #========================
        # BLOC ACTIONS TOOLS KIT
        #========================
        def openToolSearchFiles(self):
            if self.dialogSearchFiles == None :    
                    self.dialogSearchFiles = toolSearchFilesDialog(self)
                    self.windowsNoMDIChilds.append(self.dialogSearchFiles)
                    MakeWindowIcon(self.dialogSearchFiles, "toolSearchFiles.png")
            self.dialogSearchFiles.show()
            

        def openXMLEditor(self):
                d = xmlEditor(None, "" )
                MakeWindowIcon(d, "editxml.png")
                MakePropertiesForWindow(d, self)
                d.setWindowTitle("%s : %s" % (d.racTitle, ""))
                d.LoadFile()
                d.exec_()

        def openContacts(self):
                d = DialogContacts(self, False)
                d.exec_()

        def openOptions(self):
                d = DialogOptions(self.parent)
                d.exec_()
                

        #===============================
        # BLOC ACTIONS MAINWINDOW ITEMS
        #===============================
        def makeMenuCentralWidget(self, tabTarget, icon, title, listActions):
            toolBar = QToolBar(tabTarget)
	    toolBar.setObjectName(title)
	    toolBar.setAllowedAreas(Qt.TopToolBarArea | Qt.BottomToolBarArea)
	    toolBar.setWindowTitle(title)
	    toolBar.setStyleSheet("QToolBar {spacing:5px;}")
	    conteneur = None
	    
            for item in listActions :
                if item[0]=="-":  toolBar.addSeparator(); conteneur = None;
                elif item[0]=="~":
                  Obj = QGroupBox()       
                  Obj.setGeometry(item[3][0], item[3][1], item[3][2], item[3][3])
                  Obj.setMinimumSize(item[3][2], item[3][3])
                  Obj.setMaximumSize(item[3][2], item[3][3])
                  Obj.setStyleSheet("QGroupBox { font-size: 12px;  text-align: center; color : black; border: 0px;}"
                                    "QGroupBox::title {top: -2px; left: 10px;}")

                  background = QLabel(Obj)
                  background.setGeometry(10, 10, item[3][2]-20, 26)
                  background.setStyleSheet(" QLabel {background-color: qlineargradient(x1: 0, y1: 0, x2: 0, y2: 1, stop: 0 #E1E1E1, stop: 0.4 #9B9B9B, stop: 0.5 #D8D8D8, stop: 1.0 #D3D3D3);  border-width: 2px; border-radius: 10px; border-color: beige;}")

                  trumbail = QLabel(Obj)
                  trumbail.setGeometry(0, 10, 90, 25)
                  trumbail.setText("%s :" % QApplication.translate("QSphere",item[1], None, QApplication.UnicodeUTF8))
                  trumbail.setObjectName("Label%s_%s" % (item[1], item[2].split(".")[0]))
                  trumbail.setAlignment(Qt.AlignRight)
                  trumbail.setStyleSheet(" QLabel { font-size: 12px; color : black; background-color: rgba(160, 160, 160, 125); border: 0.5px solid black;  border-radius: 4px;  padding: 2px;}")

                  toolBar.addWidget(Obj)
                  conteneur = Obj

                  
                elif item[0] in ("childsCount", "windowsCount"):
                  Obj = QLabel() if not conteneur else QLabel(conteneur)      
                  Obj.setGeometry(item[3][0], item[3][1], item[3][2], item[3][3])
                  zIcon = QImage(getThemeIcon(item[2])) 
                  Obj.setPixmap(QPixmap.fromImage(zIcon))
                  Obj.setToolTip(QApplication.translate("QSphere",item[1], None, QApplication.UnicodeUTF8))

                  if not conteneur : toolBar.addWidget(Obj)

                  font = QFont()
                  font.setFamily('Courier')
                  font.setFixedPitch(True)
                  font.setPointSize(12)
                  font.setBold(True)

                  nbObj = QTextEdit() if not conteneur else QTextEdit(conteneur)
                  nbObj.setGeometry(item[3][0]+item[3][2]+5, item[3][1], item[3][2], item[3][3])
                  nbObj.setMinimumSize(item[3][2], item[3][3])
                  nbObj.setMaximumSize(item[3][2], item[3][3])
                  nbObj.setStyleSheet("QTextEdit { font-size: 14px;  text-align: center; color : white; background-color: rgba(160, 160, 160, 125); border: 2px solid cyan;  border-radius: 4px;  padding: 2px;}")
                  nbObj.setText("0")
                  nbObj.setFont(font)
                  nbObj.setAlignment(Qt.AlignCenter)
                  nbObj.setEnabled(False)
                  
                  if not conteneur : toolBar.addWidget(nbObj)

                  if item[0] == "childsCount" : self.nb_window_childs = nbObj
                  elif item[0] == "windowsCount" : self.nb_subwindows = nbObj                 
                 
                else:
                  Obj = MyPushButton() if not conteneur else MyPushButton(conteneur)

                  zText = QApplication.translate("QSphere",item[1], None, QApplication.UnicodeUTF8)
                  if item[0] == "newSession" : zText = "%s%s" % (zText, "Ctrl+Shift+N".rjust(60-len(zText)))
                  if item[0] == "openSession" : zText = "%s%s" % (zText, "Ctrl+Shift+F".rjust(60-len(zText)))
                  if item[0] == "saveSession" : zText = "%s%s" % (zText, "Ctrl+Shift+W".rjust(60-len(zText)))
                  if item[0] == "newFile" : zText = "%s%s" % (zText, "Ctrl+N".rjust(60-len(zText)))
                  if item[0] == "openFile" : zText = "%s%s" % (zText, "Ctrl+F".rjust(60-len(zText)))
                  if item[0] == "saveFile" : zText = "%s%s" % (zText, "Ctrl+W".rjust(60-len(zText)))
                  if item[0] == "saveAll" : zText = "%s%s" % (zText, "Ctrl+A".rjust(60-len(zText)))
                 
                  Obj.initPushButton(item[3][2], item[3][3], item[3][0], item[3][1], item[0], "", zText, True, getThemeIcon(item[2]), item[3][2], item[3][3], True)
                  if item[4]!= None : Obj.clicked.connect(item[4]) if type(item[4])!=str else eval(item[4])
                  if item[5]!="" : Obj.setShortcut(QKeySequence(item[5]))
                  Obj.setEnabled(item[6])

                  if item[0] =="openSession" : self.butopensession = Obj
                  elif item[0] =="openFile" : self.butopenfile = Obj
                  elif item[0] == "saveSession" : self.butsavesession = Obj
                  elif item[0] == "navigatorWeb" : self.butnavigatorweb = Obj
   
                  if item[0] in self.listNameActionsTabEdition:
                        Obj.setCheckable(True)
                        Obj.setStyleSheet(self.styleSheetPushButton)
                        if item[0] == "identificationTab" : Obj.setChecked(True)
                        self.positionTab.addButton(Obj, self.listNameActionsTabEdition.index(item[0]))

                  if item[0] in ("formMode", "xmlMode", "htmlMode"):
                        Obj.setCheckable(True)
                        Obj.setStyleSheet(self.styleSheetPushButton)
                        if item[0] == "formMode" : Obj.setChecked(True)
                        self.changeMode.addButton(Obj, ("formMode", "xmlMode", "htmlMode").index(item[0]))

                  if not conteneur : toolBar.addWidget(Obj)
                
            i = len(listActions)-1
            item = listActions[i]
          
            toolBar.setGeometry(0, 0, i*65, item[3][3]+20)   
            self.tabWidget.addTab(tabTarget, QIcon(getThemeIcon(icon)), QApplication.translate("QSphere",title, None, QApplication.UnicodeUTF8))

            return toolBar

                
        def startMovie(self):
            self.status_txt.setVisible(True)
            self.movie.start()
            self.status_txt.repaint()

        def stopMovie(self):
            self.movie.stop()
            self.status_txt.setVisible(False)

            
        def resizeEvent(self,ev):
            zSize = ev.size()
            self.tabWidget.setGeometry(0, 0, self.width(), 100)
            self.tabWidget.setStyleSheet(
            "QTabWidget::pane {height: 100px;border-top: 2px solid #C2C7CB; position: absolute; top: -0.5em; background-color: cyan;}" 
            "QTabWidget::tab-bar {alignment: center; }" 
            "QTabBar::tab:selected { border-color: #5D5D5D; border-bottom-color: #C2C7CB; border-bottom: 0px solid #C2C7CB; background-color: #f0f0f0;}"
            "QTabBar::tab {height: 64px; width: %spx; background: qlineargradient(x1: 0, y1: 0, x2: 0, y2: 1, stop: 0 #E1E1E1, stop: 0.4 #9B9B9B, stop: 0.5 #D8D8D8, stop: 1.0 #D3D3D3); border: 2px solid #C4C4C3; border-bottom-color: #C2C7CB; border-top-left-radius: 4px; border-top-right-radius: 4px; min-width: 8ex; padding: 2px;}" % (ceil(self.width()/self.tabWidget.count())-11)
            )
            self.blocProgressBar.setGeometry((self.width()/2)-205, (self.height()/2)-120, 400, 230)
            self.status_txt.setGeometry((self.width()/2)-64, (self.height()/2)-64, 64, 64)
            self.barInfo.setGeometry(0, 0, self.width(), 90)
            self.myViewSubWindows.setGeometry(self.width()-self.myViewSubWindows.width(), self.mdi.y()-self.tabWidget.height(), self.myViewSubWindows.width(), self.myViewSubWindows.height())
            

        def closeEvent(self,event):
                zTitle = QApplication.translate("QSphere","Confirm Exit...", None, QApplication.UnicodeUTF8)
                zMsg = QApplication.translate("QSphere","Are you sure you want to exit ?", None, QApplication.UnicodeUTF8)
                MyMsgBoxDialog = QMessageBox(self)
                MyMsgBoxDialog.setWindowTitle(zTitle)
                MyMsgBoxDialog.setIcon(QMessageBox.Question)
                MakeWindowIcon(MyMsgBoxDialog, "qsphere.png")

                groupBox1 = QGroupBox(MyMsgBoxDialog)
                groupBox1.setFixedSize(420, 180)
                groupBox1.setObjectName("groupBox1")

                labelText = QLabel(groupBox1)
                labelText.setText(zMsg)
                labelText.setAlignment(Qt.AlignCenter)
                labelText.setFixedSize(420, 25)
                labelText.setGeometry(5, 115, 420, 25)
                
                labelImage = QLabel(groupBox1)
                myImgPath = getThemeIcon("qsphere_stop.png")
                carIcon = QImage(myImgPath)
                labelImage.setPixmap(QPixmap.fromImage(carIcon).scaled(128, 108))
                labelImage.setFixedSize(128, 108)
                labelImage.setGeometry(145, 10, 128, 108)

                zButtonAccept = QPushButton(MyMsgBoxDialog)
                zButtonAccept.setObjectName("SaveButton")
                zButtonAccept.setText(QApplication.translate("QSphere", "Apply", None, QApplication.UnicodeUTF8))
                zButtonAccept.setFixedSize(100, 25)

                zButtonReject = QPushButton(MyMsgBoxDialog)
                zButtonReject.setObjectName("CloseButton")
                zButtonReject.setText(QApplication.translate("QSphere", "Cancel", None, QApplication.UnicodeUTF8))
                zButtonReject.setFixedSize(100, 25)

                zButtonAccept.clicked.connect(MyMsgBoxDialog.accept)
                zButtonReject.clicked.connect(MyMsgBoxDialog.reject)

                MyMsgBoxDialog.layout().addWidget(groupBox1,0,1,1,2)
                MyMsgBoxDialog.layout().addWidget(zButtonAccept,4,1,1,1)
                MyMsgBoxDialog.layout().addWidget(zButtonReject,4,2,1,1)

                MyMsgBoxDialog.setStandardButtons(MyMsgBoxDialog.NoButton)
                ret = MyMsgBoxDialog.exec_()
                if ret == 1 :
                   self.saveInfos("sessions", self.listSessions)
                   self.saveInfos("fileshistory", self.listMDDFiles)
                   self.parent.mainWindow = None
                   event.accept()
                else : event.ignore()
                        

                   
        def contextMnuSaveSessionActions(self, Obj):
                contextMnu_SaveSessionActions = QMenu()

                menuIcon = getThemeIcon("saveas.png")
                zText = QApplication.translate("QSphere","Save session as", None, QApplication.UnicodeUTF8)
                self.SaveSessionAsButton = QAction(QIcon(menuIcon), "%s ..." % zText, self)
                self.SaveSessionAsButton.setObjectName("SaveAsButton")
                contextMnu_SaveSessionActions.addAction(self.SaveSessionAsButton)
                self.SaveSessionAsButton.triggered.connect(self.saveSession)

                contextMnu_SaveSessionActions.addSeparator()

                menuIcon = getThemeIcon("savecopy.png")
                zText = QApplication.translate("QSphere","Save as a copy ...", None, QApplication.UnicodeUTF8)
                self.SaveSessionCopyAsButton = QAction(QIcon(menuIcon), zText, self)
                self.SaveSessionCopyAsButton.setObjectName("SaveCopyAsButton")
                contextMnu_SaveSessionActions.addAction(self.SaveSessionCopyAsButton)
                self.SaveSessionCopyAsButton.triggered.connect(self.saveSession)

                Obj.setMenu(contextMnu_SaveSessionActions)


        def contextMnuSaveActions(self, Obj):
                contextMnu_SaveActions = QMenu()

                menuIcon = getThemeIcon("saveas.png")
                zText = QApplication.translate("QSphere","Save as ...", None, QApplication.UnicodeUTF8)
                self.SaveAsButton = QAction(QIcon(menuIcon), zText, self)
                self.SaveAsButton.setObjectName("SaveAsButton")
                self.SaveAsButton.setShortcut(QKeySequence("Ctrl+Shift+S"))
                contextMnu_SaveActions.addAction(self.SaveAsButton)
                self.SaveAsButton.triggered.connect(self.saveAsMetadataFile)

                contextMnu_SaveActions.addSeparator()

                menuIcon = getThemeIcon("savecopy.png")
                zText = QApplication.translate("QSphere","Save as a copy ...", None, QApplication.UnicodeUTF8)
                self.SaveCopyAsButton = QAction(QIcon(menuIcon), zText, self)
                self.SaveCopyAsButton.setObjectName("SaveCopyAsButton")
                self.SaveCopyAsButton.setShortcut(QKeySequence("Ctrl+Shift+C"))
                contextMnu_SaveActions.addAction(self.SaveCopyAsButton)
                self.SaveCopyAsButton.triggered.connect(self.saveAsMetadataFile)

                contextMnu_SaveActions.addSeparator()

                menuIcon = getThemeIcon("saveattribut.png")
                zText = QApplication.translate("QSphere","Save as the Fields Map ...", None, QApplication.UnicodeUTF8)
                self.SaveXmlCatButton = QAction(QIcon(menuIcon), zText, self)
                self.SaveXmlCatButton.setObjectName("SaveXmlCatButton")
                contextMnu_SaveActions.addAction(self.SaveXmlCatButton)
                self.SaveXmlCatButton.triggered.connect(self.saveXmlCatFile)

                Obj.setMenu(contextMnu_SaveActions)

        def fixeMaxDim(self, listItems):
                maxDim = 60
                if listItems != []:
                   for item in listItems: maxDim = max(maxDim,len(item))
                maxDim = int(maxDim * 1.8)
                return maxDim
                
        def buildContextMnuFile(self, Obj):
                self.contextMnuFile = QMenu()

                maxDim = self.fixeMaxDim(self.listMDDFiles)

                menuIcon = getThemeIcon("openmeta.png")
                zText = "%s ..." % QApplication.translate("QSphere","Load a metadata file", None, QApplication.UnicodeUTF8)
                zText = "%s%s" % (zText, "Ctrl+F".rjust(maxDim-len(zText)))
                self.openFile = QAction(QIcon(menuIcon), zText, self)
                self.openFile.setObjectName("mActionOpenFile")
                #self.openFile.setShortcut(QKeySequence("Ctrl+F"))
                self.contextMnuFile.addAction(self.openFile)
                self.openFile.triggered.connect(self.openMetadataFile)

                if self.listMDDFiles != []:
                        self.contextMnuFile.addSeparator()

                        i = 0
                        for mddfile in self.listMDDFiles :
                            if mddfile :    
                                    self.actionLastFile = QAction(QIcon(menuIcon), mddfile, self)
                                    self.actionLastFile.setObjectName("actionLastFile%s" % (i) )
                                    self.contextMnuFile.addAction(self.actionLastFile)
                                    self.actionLastFile.triggered.connect(self.openLastFile)
                                    i+= 1


                self.contextMnuFile.addSeparator()

                menuIcon = getThemeIcon("poubelle.png")
                zText = "%s ..." % QApplication.translate("QSphere","Clear history", None, QApplication.UnicodeUTF8)
                self.delAllSavedFiles = QAction(QIcon(menuIcon), zText, self)
                self.delAllSavedFiles.setObjectName("delAllSavedFiles")
                self.contextMnuFile.addAction(self.delAllSavedFiles)
                self.delAllSavedFiles.triggered.connect(self.delInfos)

                Obj.setMenu(self.contextMnuFile)
                

        def buildContextMnuSession(self, Obj):
                self.contextMnuSession = QMenu()

                menuIcon = getThemeIcon("opensession.png")
                zText = "%s ..." % QApplication.translate("QSphere","Open session", None, QApplication.UnicodeUTF8)

                maxDim = self.fixeMaxDim(self.listSessions)
                
                zText = "%s%s" % (zText, "Ctrl+Shift+F".rjust(maxDim-len(zText)))
                self.OpenSession = QAction(QIcon(menuIcon), zText, self)
                self.OpenSession.setObjectName("mActionOpenSession")
                self.contextMnuSession.addAction(self.OpenSession)
                self.OpenSession.triggered.connect(self.openSession)
                
                if self.listSessions != []:
                        self.contextMnuSession.addSeparator()

                        i = 0
                        for session in self.listSessions :
                            if session :    
                                    self.actionLastSession = QAction(QIcon(menuIcon), session, self)
                                    self.actionLastSession.setObjectName("actionLastSession%s" % (i) )
                                    self.contextMnuSession.addAction(self.actionLastSession)
                                    self.actionLastSession.triggered.connect(self.openLastSession)
                                    i+= 1

                self.contextMnuSession.addSeparator()

                menuIcon = getThemeIcon("poubelle.png")
                zText = "%s ..." % QApplication.translate("QSphere","Clear history", None, QApplication.UnicodeUTF8)
                self.delAllSavedSession = QAction(QIcon(menuIcon), zText, self)
                self.delAllSavedSession.setObjectName("delAllSavedSession")
                self.contextMnuSession.addAction(self.delAllSavedSession)
                self.delAllSavedSession.triggered.connect(self.delInfos)

                Obj.setMenu(self.contextMnuSession)


        def buildContextMnuNavigatorWeb(self, Obj):
                self.contextMnuNavigatorWeb = QMenu()

                menuIcon = getThemeIcon("navigatorweb.png")
                zText = QApplication.translate("QSphere","Web navigator ...", None, QApplication.UnicodeUTF8)
                self.OpenNavigatorWeb = QAction(QIcon(menuIcon), zText, self)
                self.OpenNavigatorWeb.setObjectName("mActionNavigator")
                self.contextMnuNavigatorWeb.addAction(self.OpenNavigatorWeb)
                self.OpenNavigatorWeb.triggered.connect(self.DownloadMetadata)

                self.contextMnuNavigatorWeb.addSeparator()

                menuIcon = getThemeIcon("dmetadata.png")
                zText = QApplication.translate("QSphere","Url Server for metadata : ", None, QApplication.UnicodeUTF8)
                self.contextMnu_serverMetadata = QMenu(zText)
                self.contextMnu_serverMetadata.setIcon(QIcon(menuIcon))
                i = 0
                for elt in self.serverMetadata :
                    self.OpenServerMDD = QAction(QIcon(menuIcon), elt, self)
                    self.OpenServerMDD.setObjectName("OpenServerMDD%s" % (i) )
                    self.contextMnu_serverMetadata.addAction(self.OpenServerMDD)
                    self.OpenServerMDD.triggered.connect(self.DownloadMetadata)
                    i+= 1

                self.contextMnuNavigatorWeb.addMenu(self.contextMnu_serverMetadata)
                self.contextMnuNavigatorWeb.addSeparator()

                menuIcon = getThemeIcon("dkeywords.png")
                zText = QApplication.translate("QSphere","Url Server for keywords : ", None, QApplication.UnicodeUTF8)
                self.contextMnu_serverKeywords = QMenu(zText)
                self.contextMnu_serverKeywords.setIcon(QIcon(menuIcon))
                i = 0
                for elt in self.serverKeywords :
                    self.OpenServerKW = QAction(QIcon(menuIcon), elt, self)
                    self.OpenServerKW.setObjectName("OpenServerKW%s" % (i) )
                    self.contextMnu_serverKeywords.addAction(self.OpenServerKW)
                    self.OpenServerKW.triggered.connect(self.DownloadMetadata)
                    i+= 1

                self.contextMnuNavigatorWeb.addMenu(self.contextMnu_serverKeywords)

                self.contextMnuNavigatorWeb.addSeparator()
                
                menuIcon = getThemeIcon("info.png")
                zText = QApplication.translate("QSphere","Get informations about metadata and INSPIRE", None, QApplication.UnicodeUTF8)
                self.helptopic = QAction(QIcon(menuIcon), zText, self)
                self.helptopic.setObjectName("helptopic")
                self.contextMnuNavigatorWeb.addAction(self.helptopic)
                self.helptopic.triggered.connect(self.openHelpINSPIRE)
                
                Obj.setMenu(self.contextMnuNavigatorWeb)

        #LOAD AND SAVE HISTORY FOR FILES OR SESSIONS
        def ajustListInfos(self, historyFile, listinfos):
            if historyFile in listinfos : return    
            newHistory = []
            newHistory.append(historyFile)
            for i in range(min(len(listinfos), 9)) :
                if listinfos[i] != "" : newHistory.append(listinfos[i])
            if listinfos is self.listSessions :
               self.listSessions = list(newHistory)
               self.buildContextMnuSession(self.butopensession)
            elif listinfos is self.listMDDFiles :
               self.listMDDFiles = list(newHistory)
               self.buildContextMnuFile(self.butopenfile)
                
        def saveInfos(self, flaginfo, listinfos):
            mySettings = QSettings()
            pos = 0
            for i in range(min(len(listinfos), 10)) : 
                historyFile = listinfos[i]
                if historyFile :
                   mySettings.setValue("/qsphere/%s/" % flaginfo, pos)
                   mySettings.setValue("/qsphere/%s/%s/file" % (flaginfo, pos), "%s" % historyFile)
                   pos+= 1

        def loadInfos(self, flaginfo, listinfos):
            mySettings = QSettings()
            mySettings.beginGroup("/qsphere/%s/" % flaginfo)
            keys = mySettings.childGroups()
            if keys != []:
                   valueSettings = QSettings() 
                   for key in keys :
                       historyFile = valueSettings.value("/qsphere/%s/%s/file" % (flaginfo, key))
                       if historyFile : listinfos.append(historyFile)

        def delInfos(self):
            if self.sender().objectName()== "delAllSavedSession" : self.delSavedInfos("sessions", self.listSessions)
            elif self.sender().objectName()== "delAllSavedFiles" : self.delSavedInfos("fileshistory", self.listMDDFiles)
            
        def delSavedInfos(self, flaginfo, listinfos):
            zTitle = QApplication.translate("QSphere","Warning", None, QApplication.UnicodeUTF8)
            zMsg = QApplication.translate("QSphere","Delete all the entries ?", None, QApplication.UnicodeUTF8)
            if QMessageBox.question(None, zTitle, zMsg, QApplication.translate("QSphere","Validate", None, QApplication.UnicodeUTF8), QApplication.translate("QSphere","Cancel", None, QApplication.UnicodeUTF8)) ==  0 :
               mySettings = QSettings()
               mySettings.beginGroup("/qsphere/%s/" % flaginfo)
               keys = mySettings.childGroups()
               myValueSettings = QSettings()
               if keys != []:
                  for key in keys : myValueSettings.remove("/qsphere/%s/%s" % (flaginfo, key))
               myValueSettings.sync()
                  
               if listinfos is self.listSessions :
                  self.listSessions = []
                  self.buildContextMnuSession(self.butopensession)
               elif listinfos is self.listMDDFiles :
                  self.listMDDFiles = []     
                  self.buildContextMnuFile(self.butopenfile)
            
